#
# library for cpu related helper functions
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; specifically version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: IBM (c) 2019
# Author: Satheesh Rajendran <sathnaga@linux.vnet.ibm.com>


import re
import json
import logging
import platform
import time
import os
import xml.etree.ElementTree as ET


from avocado.utils import cpu as utils
from avocado.utils import software_manager
from avocado.utils import process
from avocado.core import exceptions
from virttest import virsh
from virttest import utils_misc
from virttest import libvirt_xml
from virttest.libvirt_xml.xcepts import LibvirtXMLNotFoundError
from virttest import libvirt_version


ARCH = platform.machine()

CPU_TYPES = {"AuthenticAMD": ["EPYC-Rome", "EPYC", "Opteron_G5",
                              "Opteron_G4", "Opteron_G3", "Opteron_G2",
                              "Opteron_G1"],
             "GenuineIntel": ["KnightsMill", "Cooperlake",
                              "Icelake-Server", "Icelake-Server-noTSX",
                              "Icelake-Client", "Icelake-Client-noTSX",
                              "Cascadelake-Server", "Cascadelake-Server-noTSX",
                              "Skylake-Server", "Skylake-Server-noTSX-IBRS",
                              "Skylake-Client", "Skylake-Client-noTSX-IBRS",
                              "Broadwell", "Broadwell-noTSX",
                              "Haswell", "Haswell-noTSX", "IvyBridge",
                              "SandyBridge", "Westmere", "Nehalem",
                              "Penryn", "Conroe"]}
CPU_TYPES_RE = {"EPYC-Rome": "rdpid,wbnoinvd,stibp,clwb,umip",
                "EPYC": "avx2,adx,bmi2,sha_ni",
                "Opteron_G5": "f16c,fma4,xop,tbm",
                "Opteron_G4": ("fma4,xop,avx,xsave,aes,sse4.2|sse4_2,"
                               "sse4.1|sse4_1,cx16,ssse3,sse4a"),
                "Opteron_G3": "cx16,sse4a",
                "Opteron_G2": "cx16",
                "Opteron_G1": "",
                "KnightsMill": "avx512_4vnniw,avx512pf,avx512er",
                "Cooperlake": "avx512_bf16,stibp,arch_capabilities,hle,rtm",
                "Icelake-Server": "avx512_vnni,la57,clflushopt,hle,rtm",
                "Icelake-Server-noTSX": "avx512_vnni,la57,clflushopt",
                "Icelake-Client": ("avx512_vpopcntdq|avx512-vpopcntdq,"
                                   "avx512vbmi,avx512_vbmi2|avx512vbmi2,hle,rtm"
                                   "gfni,vaes,vpclmulqdq,avx512_vnni,hle,rtm"),
                "Icelake-Client-noTSX": ("avx512_vpopcntdq|avx512-vpopcntdq,"
                                         "avx512vbmi,avx512_vbmi2|avx512vbmi2,"
                                         "gfni,vaes,vpclmulqdq,avx512_vnni"),
                "Cascadelake-Server": ("avx512f,avx512dq,avx512bw,avx512cd,"
                                       "avx512vl,clflushopt,avx512_vnni,hle,rtm"),
                "Cascadelake-Server-noTSX": ("avx512f,avx512dq,avx512bw,avx512cd,"
                                             "avx512vl,clflushopt,avx512_vnni"),
                "Skylake-Server": "avx512f,clwb,xgetbv1,pcid,hle,rtm",
                "Skylake-Server-noTSX-IBRS": "avx512f,clwb,xgetbv1,pcid",
                "Skylake-Client": "xgetbv1,pcid,hle,rtm",
                "Skylake-Client-noTSX-IBRS": "xgetbv1,pcid",
                "Broadwell": "adx,rdseed,3dnowprefetch,hle,rtm",
                "Broadwell-noTSX": "adx,rdseed,3dnowprefetch",
                "Haswell": "fma,avx2,movbe,hle,rtm",
                "Haswell-noTSX": "fma,avx2,movbe",
                "IvyBridge": "f16c,fsgsbase,erms",
                "SandyBridge": ("avx,xsave,aes,sse4_2|sse4.2,sse4.1|sse4_1,"
                                "cx16,ssse3"),
                "Westmere": "aes,sse4.2|sse4_2,sse4.1|sse4_1,cx16,ssse3",
                "Nehalem": "sse4.2|sse4_2,sse4.1|sse4_1,cx16,ssse3",
                "Penryn": "sse4.1|sse4_1,cx16,ssse3",
                "Conroe": "ssse3"}


class UnsupportedCPU(exceptions.TestError):
    pass


def get_cpu_xmldata(vm, options=""):
    """
    Return the vcpu count details from guest xml

    :param vm: vm object
    :param options: VM options

    :return: vcpu count details on xml
    """
    cpu_xmldata = {'current_vcpu': 0, 'vcpu': None,
                   'mtype': None, 'vcpus': None}
    # Grab a dump of the guest - if we're using the --config,
    # then get an --inactive dump.
    extra_opts = ""
    if "--config" in options or vm.is_dead():
        extra_opts = "--inactive"
    vm_xml = libvirt_xml.VMXML.new_from_dumpxml(vm.name, extra_opts)
    cpu_xmldata['mtype'] = vm_xml.os.machine
    try:
        cpu_xmldata['current_vcpu'] = int(vm_xml.current_vcpu)
    except LibvirtXMLNotFoundError:
        logging.debug("current vcpu value not present in xml, set as max value")
        cpu_xmldata['current_vcpu'] = int(vm_xml.vcpu)
    cpu_xmldata['vcpu'] = int(vm_xml.vcpu)
    return cpu_xmldata


def hotplug_supported(vm_name, mtype):
    """
    hotplug support check for ppc64le

    :param vm_name: VM name
    :param mtype: machine type

    :return: True if supported and False in all other cases
    """
    supported = False
    if "ppc64" in platform.machine():
        cmd = '{\"execute\":\"query-machines\"}'
        json_result = virsh.qemu_monitor_command(vm_name, cmd, "--pretty",
                                                 debug=False)
        try:
            result = json.loads(json_result.stdout_text)
        except Exception:
            # Failure to parse json output and default support to False
            # TODO: Handle for failure cases
            return supported
        for item in result['return']:
            try:
                if item['name'] == mtype:
                    try:
                        if item['hotpluggable-cpus'] == 'True':
                            supported = True
                    except KeyError:
                        pass
            except KeyError:
                pass
    else:
        # For now returning for other arch by default true
        supported = True
    return supported


def affinity_from_vcpuinfo(vm):
    """
    Returns list of the vcpu's affinity from
    virsh vcpuinfo output

    :param vm: VM object

    :return: affinity list of VM
    """
    output = virsh.vcpuinfo(vm.name).stdout_text.rstrip()
    affinity = re.findall('CPU Affinity: +[-y]+', output)
    total_affinity = [list(vcpu_affinity.split()[-1].strip())
                      for vcpu_affinity in affinity]
    return total_affinity


def affinity_from_xml(vm):
    """
    Returns dict of the vcpu's affinity from
    guest xml

    :param vm: VM object

    :return: dict of affinity of VM
    """
    host_cpu_count = utils.total_count() if hasattr(utils, 'total_count') else utils.total_cpus_count()
    xml_affinity_list = []
    xml_affinity = {}
    try:
        vmxml = libvirt_xml.VMXML.new_from_dumpxml(vm.name)
        xml_affinity_list = vmxml['cputune'].vcpupins
    except LibvirtXMLNotFoundError:
        logging.debug("No <cputune> element find in domain xml")
        return xml_affinity
    # Store xml_affinity_list to a dict
    for vcpu in xml_affinity_list:
        xml_affinity[vcpu['vcpu']] = cpus_string_to_affinity_list(vcpu['cpuset'],
                                                                  host_cpu_count)
    return xml_affinity


def affinity_from_vcpupin(vm, vcpu=None, options=None):
    """
    Returns dict of vcpu's affinity from virsh vcpupin output

    :param vm: VM object
    :param vcpu: virtual cpu to qeury
    :param options: --live, --current or --config
    :return: dict of affinity of VM
    """
    vcpupin_output = {}
    vcpupin_affinity = {}
    host_cpu_count = utils.total_count() if hasattr(utils, 'total_count') else utils.total_cpus_count()
    result = virsh.vcpupin(vm.name, vcpu=vcpu, options=options, debug=True)
    for vcpu in result.stdout_text.strip().split('\n')[2:]:
        # On newer version of libvirt, there is no ':' in
        # vcpupin output anymore
        vcpupin_output[int(vcpu.split()[0].rstrip(':'))] = vcpu.split()[1]
    for vcpu in vcpupin_output:
        vcpupin_affinity[vcpu] = cpus_string_to_affinity_list(vcpupin_output[vcpu], host_cpu_count)
    return vcpupin_affinity


def affinity_from_proc(vm):
    """
    Return dict of affinity from proc

    :param vm: VM object

    :return: dict of affinity of VM
    """
    pid = vm.get_pid()
    proc_affinity = {}
    vcpu_pids = []
    host_cpu_count = utils.total_count() if hasattr(utils, 'total_count') else utils.total_cpus_count()
    vcpu_pids = vm.get_vcpus_pid()
    for vcpu in range(len(vcpu_pids)):
        output = cpu_allowed_list_by_task(pid, vcpu_pids[vcpu])
        output_affinity = cpus_string_to_affinity_list(output, int(host_cpu_count))
        proc_affinity[vcpu] = output_affinity
    return proc_affinity


def get_vcpucount_details(vm, options):
    """
    To get vcpucount output

    :param vm: VM object
    :param options: options to passed to vcpucount

    :return: tuple of result and dict of vcpucount output values
    """
    vcpucount_details = {'max_config': None, 'max_live': None,
                         'cur_config': None, 'cur_live': None,
                         'guest_live': None}

    result = virsh.vcpucount(vm.name, options, ignore_status=True,
                             debug=True)
    if result.exit_status:
        logging.debug("vcpu count command failed")
        return (result, vcpucount_details)

    if options:
        stdout = result.stdout_text.strip()
        if 'guest' in options:
            vcpucount_details['guest_live'] = int(stdout)
        elif 'config' in options:
            if 'maximum' in options:
                vcpucount_details['max_config'] = int(stdout)
            else:
                vcpucount_details['cur_config'] = int(stdout)
        elif 'live' in options:
            if 'maximum' in options:
                vcpucount_details['max_live'] = int(stdout)
            else:
                vcpucount_details['cur_live'] = int(stdout)
    else:
        output = result.stdout_text.strip().split('\n')
        for item in output:
            if ('maximum' in item) and ('config' in item):
                vcpucount_details['max_config'] = int(item.split()[2].strip())
            elif ('maximum' in item) and ('live' in item):
                vcpucount_details['max_live'] = int(item.split()[2].strip())
            elif ('current' in item) and ('config' in item):
                vcpucount_details['cur_config'] = int(item.split()[2].strip())
            elif ('current' in item) and ('live' in item):
                vcpucount_details['cur_live'] = int(item.split()[2].strip())
            else:
                pass
    return (result, vcpucount_details)


def check_affinity(vm, expect_vcpupin):
    """
    Check the affinity of vcpus in various libvirt API output

    :param vm: VM object
    :param expect_vcpupin: Expected affinity details

    :return: True if affinity matches from different virsh API outputs,
             False if not
    """
    host_cpu_count = utils.total_count() if hasattr(utils, 'total_count') else utils.total_cpus_count()
    affinity_xml = affinity_from_xml(vm)
    affinity_vcpupin = affinity_from_vcpupin(vm)
    affinity_vcpuinfo = affinity_from_vcpuinfo(vm)
    result = True

    for vcpu in list(expect_vcpupin.keys()):
        expect_affinity = cpus_string_to_affinity_list(str(expect_vcpupin[vcpu]), host_cpu_count)
        # Check for vcpuinfo affinity
        if affinity_vcpuinfo[int(vcpu)] != expect_affinity:
            logging.error("CPU affinity in virsh vcpuinfo output"
                          " is unexpected")
            result = False
        # Check for vcpupin affinity
        if affinity_vcpupin[int(vcpu)] != expect_affinity:
            logging.error("Virsh vcpupin output is unexpected")
            result = False
        # Check for affinity in Domain xml
        if affinity_xml:
            if affinity_xml[vcpu] != expect_affinity:
                logging.error("Affinity in domain XML is unexpected")
                result = False
    if result:
        logging.debug("Vcpupin info check pass")
    return result


def check_vcpucount(vm, exp_vcpu, option="", guest_agent=False):
    """
    To check the vcpu count details from vcpucount API

    :param vm: VM object
    :param exp_vcpu: dict of expected vcpus
    :param option: options to vcpucount API if any
    :param guest_agest: True if need to check inside guest,guest agent present

    :return: True if exp_vcpu matches the vcpucount output, False if not
    """
    result = True
    vcpucount_result = {}
    vcpucount_option = ""
    if option == "--guest" and vm.is_alive() and guest_agent:
        vcpucount_option = "--guest"
    (vcresult, vcpucount_result) = get_vcpucount_details(vm, vcpucount_option)
    if vcresult.stderr_text:
        result = False
    if vcpucount_option == "--guest" and guest_agent:
        if vcpucount_result['guest_live'] != exp_vcpu['guest_live']:
            logging.error("Virsh vcpucount output is unexpected\nExpected: "
                          "%s\nActual: %s", exp_vcpu, vcpucount_result)
            result = False
    else:
        # Check for config option results
        if vm.is_dead():
            if (exp_vcpu['max_config'] != vcpucount_result['max_config'] or
                    exp_vcpu['cur_config'] != vcpucount_result['cur_config']):
                logging.error("Virsh vcpucount output is unexpected\nExpected"
                              ":%s\nActual:%s", exp_vcpu, vcpucount_result)
                result = False
        else:
            if (exp_vcpu['max_config'] != vcpucount_result['max_config'] or
                    exp_vcpu['max_live'] != vcpucount_result['max_live'] or
                    exp_vcpu['cur_config'] != vcpucount_result['cur_config'] or
                    exp_vcpu['cur_live'] != vcpucount_result['cur_live']):
                logging.error("Virsh vcpucount output is unexpected\n "
                              "Expected:%s\nActual:%s", exp_vcpu,
                              vcpucount_result)
                result = False
    if result:
        logging.debug("Command vcpucount check pass")
    return result


def check_vcpuinfo(vm, exp_vcpu):
    """
    To check vcpu count details from virsh vcpuinfo API

    :param vm: VM object
    :param exp_vcpu: dict of expected vcpu details

    :return: True if exp_vcpu matches the vcpuinfo output, False if not
    """
    result = True
    # Decide based on vm alive status to check actual vcpu count
    if vm.is_alive():
        idx = 'cur_live'
    else:
        idx = 'cur_config'

    affinity_vcpuinfo = affinity_from_vcpuinfo(vm)
    vcpuinfo_num = len(affinity_vcpuinfo)
    if vcpuinfo_num != exp_vcpu[idx]:
        logging.error("Vcpu number in virsh vcpuinfo is unexpected\n"
                      "Expected: %s\nActual: %s", exp_vcpu[idx], vcpuinfo_num)
        result = False
    else:
        logging.debug("Command vcpuinfo check pass")
    return result


def check_xmlcount(vm, exp_vcpu, option):
    """
    To check vcpu count details from guest XML

    :param vm: VM object
    :param exp_vcpu: dict of expected vcpu details
    :param option: VM options

    :return: True if exp_vcpu matches the vcpuinfo output, False if not
    """
    result = True
    cpu_xml = {}
    cpu_xml = get_cpu_xmldata(vm, option)
    if "--config" in option or vm.is_dead():
        exp_key = "cur_config"
    else:
        exp_key = "cur_live"
    if cpu_xml['current_vcpu'] != exp_vcpu[exp_key]:
        if cpu_xml['current_vcpu'] != exp_vcpu['cur_config']:
            logging.error("currrent vcpu number mismatch in xml\n"
                          "Expected: %s\nActual:%s", exp_vcpu[exp_key],
                          cpu_xml['current_vcpu'])
            result = False
        else:
            logging.debug("current vcpu count in xml check pass")
    if cpu_xml['vcpu'] != exp_vcpu['max_config']:
        logging.error("vcpu count mismatch in xml\nExpected: %s\nActual: %s",
                      exp_vcpu['max_config'], cpu_xml['vcpu'])
        result = False
    else:
        logging.debug("vcpu count in xml check pass")
    return result


def get_cpustats(vm, cpu=None):
    """
    Get the cpustats output of a given domain
    :param vm: VM domain
    :param cpu: Host cpu index, default all cpus
    :return: dict of cpu stats values
    result format:
    {0:[vcputime,emulatortime,cputime]
    ..
    'total':[cputime]}
     """
    host_cpu_online = utils.online_list() if hasattr(utils, 'online_list') else utils.cpu_online_list()
    cpustats = {}
    if cpu:
        cpustats[cpu] = []
        option = "--start %s --count 1" % cpu
        result = virsh.cpu_stats(vm.name, option)
        if result.exit_status != 0:
            logging.error("cpu stats command failed: %s",
                          result.stderr_text)
            return None
        output = result.stdout_text.strip().split()
        if re.match("CPU%s" % cpu, output[0]):
            cpustats[cpu] = [float(output[5]),  # vcputime
                             float(output[2]) - float(output[5]),  # emulator
                             float(output[2])]  # cputime

    else:
        for i in range(len(host_cpu_online)):
            cpustats[host_cpu_online[i]] = []
            option = "--start %s --count 1" % host_cpu_online[i]
            result = virsh.cpu_stats(vm.name, option)
            if result.exit_status != 0:
                logging.error("cpu stats command failed: %s",
                              result.stderr_text)
                return None
            output = result.stdout_text.strip().split()
            if re.match("CPU%s" % host_cpu_online[i], output[0]):
                cpustats[host_cpu_online[i]] = [float(output[5]),
                                                float(output[2]) - float(output[5]),
                                                float(output[2])]
    result = virsh.cpu_stats(vm.name, "--total")
    cpustats["total"] = []
    if result.exit_status != 0:
        logging.error("cpu stats command failed: %s",
                      result.stderr_text)
        return None
    output = result.stdout_text.strip().split()
    cpustats["total"] = [float(output[2])]  # cputime
    return cpustats


def get_domstats(vm, key):
    """
    Get VM's domstats output value for given keyword
    :param vm: VM object
    :param key: keyword for which value is needed
    :return: value string
    """
    domstats_output = virsh.domstats(vm.name)
    for item in domstats_output.stdout_text.strip().split():
        if key in item:
            return item.split("=")[1]


def check_vcpu_domstats(vm, exp_vcpu):
    """
    Check the cpu values from domstats output
    :param vm: VM object
    :param exp_vcpu: dict of expected vcpus
    :return: True if exp_vcpu matches the domstats output, False if not
    """
    status = True
    cur_vcpu = int(get_domstats(vm, "vcpu.current"))
    max_vcpu = int(get_domstats(vm, "vcpu.maximum"))
    if vm.is_alive():
        exp_cur_vcpu = exp_vcpu['cur_live']
        exp_cur_max = exp_vcpu['max_live']
    else:
        exp_cur_vcpu = exp_vcpu['cur_config']
        exp_cur_max = exp_vcpu['max_config']
    if exp_cur_vcpu != cur_vcpu:
        status = False
        logging.error("Mismatch in current vcpu in domstats output, "
                      "Expected: %s Actual: %s", exp_cur_vcpu, cur_vcpu)
    if exp_cur_max != max_vcpu:
        status = False
        logging.error("Mismatch in maximum vcpu in domstats output, Expected:"
                      " %s Actual: %s", exp_cur_max, max_vcpu)

    return status


def check_vcpu_value(vm, exp_vcpu, vcpupin=None, option="", guest_agent=False):
    """
    Check domain vcpu, including vcpucount, vcpuinfo, vcpupin, vcpu number and
    cputune in domain xml, vcpu number inside the domain.

    :param vm: VM object
    :param exp_vcpu: dict of expect vcpu number:
        exp_vcpu['max_config'] = maximum config vcpu number
        exp_vcpu['max_live'] = maximum live vcpu number
        exp_vcpu['cur_config'] = current config vcpu number
        exp_vcpu['cur_live'] = current live vcpu number
        exp_vcpu['guest_live'] = vcpu number inside the domain
    :param vcpupin: A Dict of expect vcpu affinity
    :param option: Option for virsh commands(setvcpu, setvcpus etc)
    :param guest_agent: True if agent present

    :return: True if the exp_vcpu values matches with virsh API values
            False if not
    """
    final_result = True
    logging.debug("Expect vcpu number: %s", exp_vcpu)

    # 1.1 Check virsh vcpucount output
    if not check_vcpucount(vm, exp_vcpu, option, guest_agent):
        final_result = False

    # 1.2 Check virsh vcpuinfo output
    if not check_vcpuinfo(vm, exp_vcpu):
        final_result = False

    # 1.3 Check affinity from virsh vcpupin,virsh vcpuinfo, xml(cputune)
    if vcpupin:
        if not check_affinity(vm, vcpupin):
            final_result = False

    # 1.4 Check the vcpu count in the xml
    if not check_xmlcount(vm, exp_vcpu, option):
        final_result = False

    if vm.is_alive() and (not vm.is_paused()) and "live" in option:
        vcpu_hotplug_timeout = 120  # maximum time to wait for a hotplug event to complete
        # 1.5 Check inside the guest
        if not utils_misc.wait_for(lambda: check_if_vm_vcpu_match(exp_vcpu['guest_live'],
                                                                  vm),
                                   vcpu_hotplug_timeout, text="wait for vcpu online"):
            final_result = False
        # 1.6 Check guest numa
        if not guest_numa_check(vm, exp_vcpu):
            final_result = False
    # 1.7 Check virsh domstats output
    if not check_vcpu_domstats(vm, exp_vcpu):
        final_result = False

    return final_result


def is_qemu_kvm_ma():
    """
    Check if qemu-kvm-ma is installed in host
    """
    sm = software_manager.SoftwareManager()
    return sm.check_installed("qemu-kvm-ma")


def vcpuhotunplug_unsupport_str():
    """
    Check if qemu-kvm-ma is installed and return unsupport err string
    """
    if is_qemu_kvm_ma():
        return "not currently supported"
    else:
        return ""


def guest_numa_check(vm, exp_vcpu):
    """
    To check numa node values

    :param vm: VM object
    :param exp_vcpu: dict of expected vcpus
    :return: True if check succeed, False otherwise
    """
    logging.debug("Check guest numa")
    session = vm.wait_for_login()
    vm_cpu_info = get_cpu_info(session)
    session.close()
    vmxml = libvirt_xml.VMXML.new_from_dumpxml(vm.name)
    try:
        node_num_xml = len(vmxml.cpu.numa_cell)
    except (TypeError, LibvirtXMLNotFoundError):
        # handle if no numa cell in guest xml, bydefault node 0
        node_num_xml = 1
    node_num_guest = int(vm_cpu_info["NUMA node(s)"])
    exp_num_nodes = node_num_xml
    status = True
    for node in range(node_num_xml):
        try:
            node_cpu_xml = vmxml.cpu.numa_cell[node]['cpus']
            node_cpu_xml = cpus_parser(node_cpu_xml)
        except (TypeError, LibvirtXMLNotFoundError):
            try:
                node_cpu_xml = vmxml.current_vcpu
            except LibvirtXMLNotFoundError:
                node_cpu_xml = vmxml.vcpu
            node_cpu_xml = list(range(int(node_cpu_xml)))
        try:
            node_mem_xml = vmxml.cpu.numa_cell[node]['memory']
        except (TypeError, LibvirtXMLNotFoundError):
            node_mem_xml = vmxml.memory
        node_mem_guest = int(vm.get_totalmem_sys(node=node))
        node_cpu_xml_copy = node_cpu_xml[:]
        for cpu in node_cpu_xml_copy:
            if int(cpu) >= int(exp_vcpu["guest_live"]):
                node_cpu_xml.remove(cpu)
        if (not node_cpu_xml) and node_mem_guest == 0:
            exp_num_nodes -= 1
        try:
            node_cpu_guest = vm_cpu_info["NUMA node%s CPU(s)" % node]
            node_cpu_guest = cpus_parser(node_cpu_guest)
        except KeyError:
            node_cpu_guest = []
        # Check cpu
        if node_cpu_xml != node_cpu_guest:
            status = False
            logging.error("Mismatch in cpus in node %s: xml %s guest %s", node,
                          node_cpu_xml, node_cpu_guest)
        # Check memory
        if int(node_mem_xml) != node_mem_guest:
            status = False
            logging.error("Mismatch in memory in node %s: xml %s guest %s", node,
                          node_mem_xml, node_mem_guest)
    # Check no. of nodes
    if exp_num_nodes != node_num_guest:
        status = False
        logging.error("Mismatch in numa nodes expected nodes: %s guest: %s", exp_num_nodes,
                      node_num_guest)
    return status


def get_load_per(session=None, iterations=2, interval=10.0):
    """
    Get the percentage of load in Guest/Host
    :param session: Guest Sesssion
    :param iterations: iterations
    :param interval: interval between load calculation
    :return: load of system in percentage
    """
    idle_secs = []
    idle_per = []
    cmd = "cat /proc/uptime"
    for itr in range(iterations):
        if session:
            idle_secs.append(float(session.cmd_output(cmd).strip().split()[1]))
        else:
            idle_secs.append(float(process.system_output(cmd).split()[1]))
        time.sleep(interval)
    for itr in range(iterations - 1):
        idle_per.append((idle_secs[itr + 1] - idle_secs[itr]) / interval)
    return int((1 - (sum(idle_per) / len(idle_per))) * 100)


def get_thread_cpu(thread):
    """
    Get the light weight process(thread) used cpus.

    :param thread: thread checked
    :type thread: string
    :return: A list include all cpus the thread used
    :rtype: builtin.list
    """
    cmd = "ps -o cpuid,lwp -eL | grep -w %s$" % thread
    cpu_thread = process.run(cmd, shell=True).stdout_text
    if not cpu_thread:
        return []
    return list(set([_.strip().split()[0] for _ in cpu_thread.splitlines()]))


def get_pid_cpu(pid):
    """
    Get the process used cpus.

    :param pid: process id
    :type thread: string
    :return: A list include all cpus the process used
    :rtype: builtin.list
    """
    cmd = "ps -o cpuid -L -p %s" % pid
    cpu_pid = process.run(cmd).stdout_text
    if not cpu_pid:
        return []
    return list(set([_.strip() for _ in cpu_pid.splitlines()]))


def get_cpu_info(session=None):
    """
    Return information about the CPU architecture

    :param session: session Object
    :return: A dirt of cpu information
    """
    cpu_info = {}
    cmd = "lscpu"
    if session is None:
        output = process.run(cmd, ignore_status=True).stdout_text.splitlines()
    else:
        try:
            output = session.cmd_output(cmd).splitlines()
        finally:
            session.close()
    cpu_info = dict(map(lambda x: [i.strip() for i in x.split(":")], output))
    return cpu_info


class Flag(str):

    """
    Class for easy merge cpuflags.
    """
    aliases = {}

    def __new__(cls, flag):
        if flag in Flag.aliases:
            flag = Flag.aliases[flag]
        return str.__new__(cls, flag)

    def __eq__(self, other):
        s = set(self.split("|"))
        o = set(other.split("|"))
        if s & o:
            return True
        else:
            return False

    def __str__(self):
        return self.split("|")[0]

    def __repr__(self):
        return self.split("|")[0]

    def __hash__(self, *args, **kwargs):
        return 0


kvm_map_flags_to_test = {
    Flag('avx'): set(['avx']),
    Flag('sse3|pni'): set(['sse3']),
    Flag('ssse3'): set(['ssse3']),
    Flag('sse4.1|sse4_1|sse4.2|sse4_2'): set(['sse4']),
    Flag('aes'): set(['aes', 'pclmul']),
    Flag('pclmuldq'): set(['pclmul']),
    Flag('pclmulqdq'): set(['pclmul']),
    Flag('rdrand'): set(['rdrand']),
    Flag('sse4a'): set(['sse4a']),
    Flag('fma4'): set(['fma4']),
    Flag('xop'): set(['xop']),
}


kvm_map_flags_aliases = {
    'sse4_1': 'sse4.1',
    'sse4_2': 'sse4.2',
    'pclmuldq': 'pclmulqdq',
    'sse3': 'pni',
    'ffxsr': 'fxsr_opt',
    'xd': 'nx',
    'i64': 'lm',
    'psn': 'pn',
    'clfsh': 'clflush',
    'dts': 'ds',
    'htt': 'ht',
    'CMPXCHG8B': 'cx8',
    'Page1GB': 'pdpe1gb',
    'LahfSahf': 'lahf_lm',
    'ExtApicSpace': 'extapic',
    'AltMovCr8': 'cr8_legacy',
    'cr8legacy': 'cr8_legacy'
}


def kvm_flags_to_stresstests(flags):
    """
    Covert [cpu flags] to [tests]

    :param cpuflags: list of cpuflags
    :return: Return tests like string.
    """
    tests = set([])
    for f in flags:
        tests |= kvm_map_flags_to_test[f]
    param = ""
    for f in tests:
        param += "," + f
    return param


def get_cpu_flags(cpu_info=""):
    """
    Returns a list of the CPU flags
    """
    cpu_flags_re = "flags\s+:\s+([\w\s]+)\n"
    if not cpu_info:
        fd = open("/proc/cpuinfo")
        cpu_info = fd.read()
        fd.close()
    cpu_flag_lists = re.findall(cpu_flags_re, cpu_info)
    if not cpu_flag_lists:
        return []
    cpu_flags = cpu_flag_lists[0]
    return re.split("\s+", cpu_flags.strip())


def get_cpu_vendor(cpu_info="", verbose=True):
    """
    Returns the name of the CPU vendor
    """
    vendor_re = "vendor_id\s+:\s+(\w+)"
    if not cpu_info:
        fd = open("/proc/cpuinfo")
        cpu_info = fd.read()
        fd.close()
    vendor = re.findall(vendor_re, cpu_info)
    if not vendor:
        vendor = 'unknown'
    else:
        vendor = vendor[0]
    if verbose:
        logging.debug("Detected CPU vendor as '%s'", vendor)
    return vendor


def get_recognized_cpuid_flags(qemu_binary="/usr/libexec/qemu-kvm"):
    """
    Get qemu recongnized CPUID flags

    :param qemu_binary: qemu-kvm binary file path
    :return: flags list
    """
    out = process.run("%s -cpu ?" % qemu_binary).stdout.decode(errors='replace')
    match = re.search("Recognized CPUID flags:(.*)", out, re.M | re.S)
    try:
        return list(filter(None, re.split('\s', match.group(1))))
    except AttributeError:
        pass
    return []


def get_host_cpu_models():
    """
    Get cpu model from host cpuinfo
    """
    def _cpu_flags_sort(cpu_flags):
        """
        Update the cpu flags get from host to a certain order and format
        """
        flag_list = sorted(re.split("\s+", cpu_flags.strip()))
        cpu_flags = " ".join(flag_list)
        return cpu_flags

    def _make_up_pattern(flags):
        """
        Update the check pattern to a certain order and format
        """
        pattern_list = sorted(re.split(",", flags.strip()))
        pattern = r"(\b%s\b)" % pattern_list[0]
        for i in pattern_list[1:]:
            pattern += r".+(\b%s\b)" % i
        return pattern

    if ARCH in ('ppc64', 'ppc64le'):
        return []     # remove -cpu and leave it on qemu to decide

    fd = open("/proc/cpuinfo")
    cpu_info = fd.read()
    fd.close()

    cpu_flags = " ".join(get_cpu_flags(cpu_info))
    vendor = get_cpu_vendor(cpu_info)

    cpu_model = None
    cpu_support_model = []
    if cpu_flags:
        cpu_flags = _cpu_flags_sort(cpu_flags)
        for cpu_type in CPU_TYPES.get(vendor):
            pattern = _make_up_pattern(CPU_TYPES_RE.get(cpu_type))
            if re.findall(pattern, cpu_flags):
                cpu_model = cpu_type
                cpu_support_model.append(cpu_model)
    else:
        logging.warn("Can not Get cpu flags from cpuinfo")

    return cpu_support_model


def extract_qemu_cpu_models(qemu_cpu_help_text):
    """
    Get all cpu models from qemu -cpu help text.

    :param qemu_cpu_help_text: text produced by <qemu> -cpu '?'
    :return: list of cpu models
    """
    def check_model_list(pattern):
        cpu_re = re.compile(pattern)
        qemu_cpu_model_list = cpu_re.findall(qemu_cpu_help_text)
        if qemu_cpu_model_list:
            return qemu_cpu_model_list
        else:
            return None

    x86_pattern_list = "x86\s+\[?([a-zA-Z0-9_-]+)\]?.*\n"
    ppc64_pattern_list = "PowerPC\s+\[?([a-zA-Z0-9_-]+\.?[0-9]?)\]?.*\n"
    s390_pattern_list = "s390\s+\[?([a-zA-Z0-9_-]+)\]?.*\n"

    for pattern_list in [x86_pattern_list, ppc64_pattern_list, s390_pattern_list]:
        model_list = check_model_list(pattern_list)
        if model_list is not None:
            return model_list

    e_msg = ("CPU models reported by qemu -cpu ? not supported by avocado-vt. "
             "Please work with us to add support for it")
    logging.error(e_msg)
    for line in qemu_cpu_help_text.splitlines():
        logging.error(line)
    raise UnsupportedCPU(e_msg)


def check_if_vm_vcpu_match(vcpu_desire, vm, connect_uri=None, session=None):
    """
    This checks whether the VM vCPU quantity matches
    the value desired.

    :param vcpu_desire: vcpu value to be checked
    :param vm: VM Object
    :param connect_uri: libvirt uri of target host
    :param session: ShellSession object of VM

    :return: Boolean, True if actual vcpu value matches with vcpu_desire
    """
    release = vm.get_distro(connect_uri=connect_uri)
    if release and release in ['fedora', ]:
        vcpu_actual = vm.get_cpu_count("cpu_chk_all_cmd",
                                       connect_uri=connect_uri)
    else:
        vcpu_actual = vm.get_cpu_count("cpu_chk_cmd",
                                       connect_uri=connect_uri)
    if isinstance(vcpu_desire, str) and vcpu_desire.isdigit():
        vcpu_desire = int(vcpu_desire)
    if vcpu_desire != vcpu_actual:
        logging.debug("CPU quantity mismatched !!! guest said it got %s "
                      "but we assigned %s" % (vcpu_actual, vcpu_desire))
        return False
    logging.info("CPU quantity matched: %s" % vcpu_actual)
    return True


def get_model_features(model_name):
    """
    libvirt-4.5.0 :/usr/share/libvirt/cpu_map.xml defines all CPU models.
    libvirt-5.0.0 :/usr/share/libvirt/cpu_map/ defines all CPU models.
    One CPU model is a set of features.
    This function is to get features of one specific model.

    :params model_name: CPU model name, valid name is given in cpu_map.xml
    :return: feature list, like ['apic', 'ss']

    """
    features = []
    conf = "/usr/share/libvirt/cpu_map.xml"
    conf_dir = "/usr/share/libvirt/cpu_map/"

    try:
        if not libvirt_version.version_compare(5, 0, 0):
            with open(conf, 'r') as output:
                root = ET.fromstring(output.read())
                while True:
                    # Find model in file /usr/share/libvirt/cpu_map.xml
                    for model_n in root.findall('arch/model'):
                        if model_n.get('name') == model_name:
                            model_node = model_n
                            for feature in model_n.findall('feature'):
                                features.append(feature.get('name'))
                            break
                    # Handle nested model
                    if model_node.find('model') is not None:
                        model_name = model_node.find('model').get('name')
                        continue
                    else:
                        break

        else:
            # Find model in dir /usr/share/libvirt/cpu_map
            filelist = os.listdir(conf_dir)
            for file_name in filelist:
                if model_name in file_name:
                    with open(os.path.join(conf_dir, file_name), "r") as output:
                        model = ET.fromstring(output.read())
                        for feature in model.findall("model/feature"):
                            features.append(feature.get('name'))
                        break
    except ET.ParseError as error:
        logging.warn("Configuration file %s has wrong xml format" % conf)
        raise
    except AttributeError as elem_attr:
        logging.warn("No attribute %s in file %s" % (str(elem_attr), conf))
        raise
    except Exception:
        # Other excptions like IOError when open/read configuration file,
        # capture here
        logging.warn("Some other exceptions, like configuration file is not "
                     "found or not file: %s" % conf)
        raise

    return features


def cpus_string_to_affinity_list(cpus_string, num_cpus):
    """
    Parse the cpus_string string to a affinity list.

    e.g
    host_cpu_count = 4
    0       -->     [y,-,-,-]
    0,1     -->     [y,y,-,-]
    0-2     -->     [y,y,y,-]
    0-2,^2  -->     [y,y,-,-]
    r       -->     [y,y,y,y]
    """
    # Check the input string.
    single_pattern = r"\d+"
    between_pattern = r"\d+-\d+"
    exclude_pattern = r"\^\d+"
    sub_pattern = r"(%s)|(%s)|(%s)" % (exclude_pattern,
                                       single_pattern, between_pattern)
    pattern = r"^((%s),)*(%s)$" % (sub_pattern, sub_pattern)
    if not re.match(pattern, cpus_string):
        logging.debug("Cpus_string=%s is not a supported format for cpu_list."
                      % cpus_string)
    # Init a list for result.
    affinity = []
    for i in range(int(num_cpus)):
        affinity.append('-')
    # Letter 'r' means all cpus.
    if cpus_string == "r":
        for i in range(len(affinity)):
            affinity[i] = "y"
        return affinity
    # Split the string with ','.
    sub_cpus = cpus_string.split(",")
    # Parse each sub_cpus.
    for cpus in sub_cpus:
        if "-" in cpus:
            minmum = cpus.split("-")[0]
            maxmum = cpus.split("-")[-1]
            for i in range(int(minmum), int(maxmum) + 1):
                affinity[i] = "y"
        elif "^" in cpus:
            affinity[int(cpus.strip("^"))] = "-"
        else:
            affinity[int(cpus)] = "y"
    return affinity


def cpu_allowed_list_by_task(pid, tid):
    """
    Get the Cpus_allowed_list in status of task.
    """
    cmd = "cat /proc/%s/task/%s/status|grep Cpus_allowed_list:| awk '{print $2}'" % (
        pid, tid)
    result = process.run(cmd, ignore_status=True, shell=True)
    if result.exit_status:
        return None
    return result.stdout_text.strip()


def hotplug_domain_vcpu(vm, count, by_virsh=True, hotplug=True):
    """
    Hot-plug/Hot-unplug vcpu for domian

    :param vm:   VM object
    :param count:    to setvcpus it's the current vcpus number,
                     but to qemu-monitor-command,
                     we need to designate a specific CPU ID.
                     The default will be got by (count - 1)
    :param by_virsh: True means hotplug/unplug by command setvcpus,
                     otherwise, using qemu_monitor
    :param hotplug:  True means hot-plug, False means hot-unplug
    """
    if by_virsh:
        result = virsh.setvcpus(vm.name, count, "--live", debug=True)
    else:
        cmds = []
        cmd_type = "--hmp"
        result = None
        if "ppc" in platform.machine():
            vmxml = libvirt_xml.VMXML.new_from_inactive_dumpxml(vm.name)
            topology = vmxml.get_cpu_topology()
            vcpu_count = vm.get_cpu_count()

            if topology:
                threads = int(topology["threads"])
            else:
                threads = 1
            # test if count multiple of threads
            err_str = "Expected vcpu counts to be multiples of %d" % threads
            if hotplug:
                err_str += ",Invalid vcpu counts for hotplug"
            else:
                err_str += ",Invalid vcpu counts for hotunplug"
            if (count % threads) != 0:
                raise exceptions.TestError(err_str)
            if hotplug:
                for item in range(0, int(count), threads):
                    if item < vcpu_count:
                        continue
                    cmds.append("device_add host-spapr-cpu-core,id=core%d,core-id=%d" % (item, item))
            else:
                for item in range(int(count), vcpu_count, threads):
                    cmds.append("device_del core%d" % item)
        else:
            cmd_type = "--pretty"
            if hotplug:
                cpu_opt = "cpu-add"
            else:
                cpu_opt = "cpu-del"
                # Note: cpu-del is supported currently, it will return error.
                # as follow,
                # {
                #    "id": "libvirt-23",
                #    "error": {
                #        "class": "CommandNotFound",
                #        "desc": "The command cpu-del has not been found"
                #    }
                # }
                # so, the caller should check the result.
            # hot-plug/hot-plug the CPU has maximal ID
            params = (cpu_opt, (count - 1))
            cmds.append('{\"execute\":\"%s\",\"arguments\":{\"id\":%d}}' % params)
        # Execute cmds to hot(un)plug
        for cmd in cmds:
            result = virsh.qemu_monitor_command(vm.name, cmd, cmd_type,
                                                debug=True)
            if result.exit_status != 0:
                raise exceptions.TestFail(result.stderr_text)
            else:
                logging.debug("Command output:\n%s",
                              result.stdout_text.strip())
    return result


def cpus_parser(cpulist):
    """
    Parse a list of cpu list, its syntax is a comma separated list,
    with '-' for ranges and '^' denotes exclusive.
    :param cpulist: a list of physical CPU numbers
    """
    hyphens = []
    carets = []
    commas = []
    others = []

    if cpulist is None:
        return None

    else:
        if "," in cpulist:
            cpulist_list = re.split(",", cpulist)
            for cpulist in cpulist_list:
                if "-" in cpulist:
                    tmp = re.split("-", cpulist)
                    hyphens = hyphens + list(range(int(tmp[0]), int(tmp[-1]) + 1))
                elif "^" in cpulist:
                    tmp = re.split("\^", cpulist)[-1]
                    carets.append(int(tmp))
                else:
                    try:
                        commas.append(int(cpulist))
                    except ValueError:
                        logging.error("The cpulist has to be an "
                                      "integer. (%s)", cpulist)
        elif "-" in cpulist:
            tmp = re.split("-", cpulist)
            hyphens = list(range(int(tmp[0]), int(tmp[-1]) + 1))
        elif "^" in cpulist:
            tmp = re.split("^", cpulist)[-1]
            carets.append(int(tmp))
        else:
            try:
                others.append(int(cpulist))
                return others
            except ValueError:
                logging.error("The cpulist has to be an "
                              "integer. (%s)", cpulist)

        cpus_set = set(hyphens).union(set(commas)).difference(set(carets))

        return sorted(list(cpus_set))


def get_qemu_cpu_models(qemu_binary):
    """Get listing of CPU models supported by QEMU
    Get list of CPU models by parsing the output of <qemu> -cpu '?'
    """
    cmd = qemu_binary + " -cpu '?'"
    result = process.run(cmd, verbose=False)
    return extract_qemu_cpu_models(result.stdout_text)


def get_qemu_best_cpu_model(params):
    """
    Try to find out the best CPU model available for qemu.

    This function can't be in qemu_vm, because it is used in env_process,
    where there's no vm object available yet, and env content is synchronized
    in multi host testing.

    1) Get host CPU model
    2) Verify if host CPU model is in the list of supported qemu cpu models
    3) If so, return host CPU model
    4) If not, return the default cpu model set in params, if none defined,
        return 'qemu64'.
    """
    host_cpu_models = get_host_cpu_models()
    qemu_binary = utils_misc.get_qemu_binary(params)
    qemu_cpu_models = get_qemu_cpu_models(qemu_binary)
    # Let's try to find a suitable model on the qemu list
    for host_cpu_model in host_cpu_models:
        if host_cpu_model in qemu_cpu_models:
            return host_cpu_model
    # If no host cpu model can be found on qemu_cpu_models, choose the default
    return params.get("default_cpu_model", None)



import pygtk
pygtk.require('2.0')
import gtk
import sys
import os
from optparse import OptionParser

from six.moves import xrange


def return_nonrandom_str(length):
    """
    A method to return a nonrandom_string of the length determined by the input

    :param length, length of the string to be returned
    """
    input_string = "anonrandomstring_"
    finalstring = ""
    str_len = len(input_string)

    for i in xrange(int(length)):
        finalstring += input_string[i % str_len]

    return finalstring


def main(argv):
    '''
    usage to set image file in clipboard:     python clipboard.py --set <path>
    usage to get image file in clipboard:     python clipboard.py
    '''

    # Command line parser alternative
    parser = OptionParser()
    parser.add_option("-c", "--clear", dest="clear",
                      action="store_true", help="clear the clipboard")
    parser.add_option("-s", "--set", dest="setcb",
                      default='', help="Sets the local clip board")
    parser.add_option("-p", "--print", dest="setcb",
                      default='', help="Prints the text that was set")
    parser.add_option("-i", "--set_image", dest="setcbimage",
                      default='', help="Sets an image to local clip board")
    parser.add_option("-m", help="Saves an image to the /tmp directory",
                      dest="setfilename")
    parser.add_option("-r", "--file_copy", dest="setcb_file",
                      default='', help="Creates a file, Reads the text from " +
                      "a file and puts it onto the clipboard")
    parser.add_option("-f", "--file_paste", dest="pastecb_filename",
                      default='', help="Given a filename, the clipboard " +
                      "contents will be pasted")
    parser.add_option("-n", "--create_file", dest="setcb_lrgstr_file",
                      default='', help="Given a integer value a file will " +
                      "be created and put on the clipboard")

    # Read the command line parameters now
    (options, _) = parser.parse_args()

    # Get the clipboard
    clipboard = gtk.clipboard_get()

    if options.setcb:

        ssetcb = options.setcb
        # Set the clipboard text data
        clipboard.set_text(ssetcb)

        # Make our data available to other applicatons
        clipboard.store()
        print('The text has been placed into the clipboard.')
    elif options.setfilename:
        # Read the clipboard image data.

        # Is there an image
        print("Is there an image: " + str(clipboard.wait_is_image_available()))

        image = clipboard.wait_for_image()
        if image is None:
            print('No image stored')
        else:
            location_to_save = options.setfilename
            image.save(location_to_save, 'png')
            print('Cb Image stored and saved to: ' + options.setfilename)
    elif options.setcbimage:
        ssetcb = options.setcbimage
        print(ssetcb)

        # Set the clipboard text data
        assert os.path.exists(ssetcb), "file does not exist"
        image = gtk.image_new_from_file(ssetcb).get_pixbuf()
        clipboard.set_image(image)

        # Make our data available to other applications
        clipboard.store()
        print('The image has been placed into the clipboard.')
    elif options.clear:
        # Get and clear the clipboard
        clipboard.set_text("")
        clipboard.store()
        print('The clipboard has been cleared.')
    elif options.setcb_file:
        # Read a file and put the contents into the clipboard
        file_contents = open(options.setcb_file, 'r')
        clipboard.set_text(file_contents.read())
        file_contents.close()
        clipboard.store()
    elif options.pastecb_filename:
        # Get the text from the clipboard and write to a file
        print('Getting the text from the clipboard')
        clipboard_text = clipboard.wait_for_text()
        print('Starting to write the clipboard text to the file' +
              options.pastecb_filename)
        file_contents = open(options.pastecb_filename, 'w')
        file_contents.write(clipboard_text)
        print('Writing of the clipboard text is complete')
        file_contents.close()
    elif options.setcb_lrgstr_file:
        # Create a non-random string of the size specified and write it to
        # a file and put it on the clipboard.
        finalstring = return_nonrandom_str(int(options.setcb_lrgstr_file))
        file_contents = open("/tmp/StringLengthTest.txt", 'w')
        file_contents.write(finalstring)
        file_contents.close()
        file_contents = open("/tmp/StringLengthTest.txt", 'r')
        clipboard.set_text(file_contents.read())
        file_contents.close()
        print("Non random string put into file: /tmp/StringLengthTest. " +
              " The string has also been placed in the clipboard. " +
              "String of size " + options.setcb_lrgstr_file)
        clipboard.store()
    else:
        # Read the clipboard text data.
        text = clipboard.wait_for_text()
        print('clipboard=' + str(text))


if __name__ == "__main__":
    main(sys.argv[1:])
    sys.exit(0)



"""
Base classes supporting Libvirt Sandbox (lxc) container testing

:copyright: 2013 Red Hat Inc.
"""

import logging
import signal

import aexpect

from six.moves import xrange


class SandboxException(Exception):

    """
    Basic exception class for problems occurring in SandboxBase or subclasses
    """

    def __init__(self, message):
        super(SandboxException, self).__init__()
        self.message = message

    def __str__(self):
        return self.message


# This is to allow us to alter back-end session management w/o affecting
# sandbox subclasses
class SandboxSession(object):

    """
    Connection instance to asynchronous I/O redirector process
    """

    # Assist with warning on re-use
    used = False

    def __init__(self):
        self.session = None  # createdby new_session

    @property
    def connected(self):
        """
        Represents True/False value if background process was created/opened
        """
        if self.session is None:
            return False
        else:
            return True

    @property
    def session_id(self):
        """
        Returns unique & persistent identifier for the background process
        """
        if self.connected:
            return self.session.get_id()
        else:
            raise SandboxException("Can't get id of non-running sandbox "
                                   "session")

    def new_session(self, command):
        """
        Create and set new opaque session object
        """
        # Allow this to be called more than once w/o consequence
        self.close_session(warn_if_nonexist=self.used)
        self.session = aexpect.Expect(command, auto_close=False)
        self.used = True

    def open_session(self, a_id):
        """
        Restore connection to existing session identified by a_id
        """
        # Allow this to be called more than once w/o consequence
        self.close_session(warn_if_nonexist=self.used)
        aexpect.Expect(a_id=a_id)
        self.used = True

    def close_session(self, warn_if_nonexist=True):
        """
        Finalize assigned opaque session object
        """
        # Allow this to be called more than once w/o consequence
        if self.connected:
            self.session.close()
        else:
            if warn_if_nonexist:
                logging.warning("Closing nonexisting sandbox session")

    def kill_session(self, sig=signal.SIGTERM):
        """
        Send a signal to the opaque session object
        """
        if self.connected:
            self.session.kill(sig=sig)
        else:
            raise SandboxException("Can't send signal to inactive sandbox "
                                   "session")

    def send(self, a_string):
        """Send a_string to session"""
        if self.connected:
            self.session.send(a_string)
        else:
            raise SandboxException("Can't send to an inactive sandbox session")

    def recv(self):
        """Return combined stdout/stderr output received so far"""
        if self.connected:
            return self.session.get_output()
        else:
            raise SandboxException("Can't get output from finalized sandbox "
                                   "session")

    def recvout(self):
        """Return just stdout output"""
        # FIXME: aexpect combines stdout and stderr in a single pipe :(
        raise NotImplementedError

    def recverr(self):
        """Return just stderr output"""
        # FIXME: aexpect combines stdout and stderr in a single pipe :(
        raise NotImplementedError

    def exit_code(self):
        """Block, and return exit code from session"""
        if self.connected:
            return self.session.get_status()
        else:
            raise SandboxException("Can't get exit code from finalized sandbox "
                                   "session")

    def is_running(self):
        """Return True if exit_code() would block"""
        if self.connected:
            return self.session.is_alive()
        else:
            return None

    def auto_clean(self, boolean):
        """Make session cleanup on GC if True"""
        if self.connected:
            self.session.auto_close = boolean
        else:
            raise SandboxException("Can't set auto_clean on disconnected "
                                   "sandbox session")


class SandboxBase(object):

    """
    Base operations for sandboxed command
    """

    # Provide unique instance number for each sandbox
    instances = None

    def __init__(self, params):
        """
        Create a new sandbox interface instance based on this type from params
        """
        # Un-pickling instances doesn't call init again
        if self.__class__.instances is None:
            self.__class__.instances = 1
        else:
            self.__class__.instances += 1
        # store a copy for use to avoid referencing class attribute
        self.identifier = self.__class__.instances
        # Allow global 'lvsb_*' keys to be overridden for specific subclass
        self.params = params.object_params(self.__class__.__name__)
        self.options = None  # opaque value consumed by make_command()
        # Aexpect has some well hidden bugs, private attribute hides
        # interface in case it changes from fixes or gets swapped out
        # entirely.
        self._session = SandboxSession()

    # Allow running sandboxes to persist across multiple tests if needed
    def __getstate__(self):
        """Serialize instance for pickling"""
        # Regular dictionary format for now, but could change later
        state = {'params': self.params,
                 'identifier': self.identifier,
                 'options': self.options}
        # Critical info. to re-connect to session when un-pickle
        if self._session.connected:
            state['session_id'] = self._session.session_id
        return state

    def __setstate__(self, state):
        """Actualize instance from state"""
        for key in ('identifier', 'params', 'options'):
            setattr(self, key, state[key])
        if 'session_id' in state:
            self._session = SandboxSession()
            self._session.open_session(state['session_id'])

    def run(self, extra=None):
        """
        Launch new sandbox as asynchronous background sandbox process

        :param extra: String of extra command-line to use but not store
        """
        sandbox_cmdline = self.make_sandbox_command_line(extra)
        logging.debug("Launching %s", sandbox_cmdline)
        self._session.new_session(sandbox_cmdline)

    def stop(self):
        """Destroy but don't finalize asynchronous background sandbox process"""
        self._session.kill_session()

    def fini(self):
        """
        Finalize asynchronous background sandbox process (destroys state!)
        """
        self._session.close_session()

    def send(self, data):
        """Send data to asynchronous background sandbox process"""
        self._session.send(data)

    def recv(self):
        """
        Return stdout and stderr from asynchronous background sandbox process
        """
        return self._session.recv()

    def recvout(self):
        """
        Return only stdout from asynchronous background sandbox process
        """
        return self._session.recvout()

    def recverr(self):
        """
        return only stderr from asynchronous background sandbox process
        """
        return self._session.recverr()

    def running(self):
        """
        Return True/False if asynchronous background sandbox process executing
        """
        return self._session.is_running()

    def exit_code(self):
        """
        Block until asynchronous background sandbox process ends, returning code
        """
        return self._session.exit_code()

    def auto_clean(self, boolean):
        """
        Change behavior of asynchronous background sandbox process on __del__
        """
        self._session.auto_clean(boolean)

    def make_sandbox_command_line(self, extra=None):
        """
        Return the fully formed command-line for the sandbox using self.options
        """
        # These are the abstract methods subclasses must override
        raise NotImplementedError


class SandboxCommandBase(SandboxBase):

    """
    Connection to a single new or existing sandboxed command
    """

    BINARY_PATH_PARAM = 'virt_sandbox_binary'

    # Cache generated name first time it is requested
    _name = None

    def __init__(self, params, name=None):
        """
        Initialize sandbox-command with params and name, autogenerate if None
        """
        if name is not None:
            self._name = name
        super(SandboxCommandBase, self).__init__(params)

    def __getstate__(self):
        """Serialize instance for pickling"""
        state = super(SandboxCommandBase, self).__getstate__()
        state['name'] = self._name
        return state

    def __setstate__(self, state):
        """Actualize instance from state"""
        self._name = state.pop('name')
        super(SandboxCommandBase, self).__setstate__(state)

    def _get_name(self):
        """
        Represent a unique sandbox name generated from class and identifier
        """
        # Use shortest possible unique names for instances to be easier
        # to track and make name-comparison fast when there are 10000's
        # of sandboxes. Only use upper-case letters from class name along
        # with instance identifier attribute.
        if self._name is None:
            class_name = self.__class__.__name__
            class_initials = class_name.translate(None,
                                                  'abcdefghijklmnopqrstuvwxyz')
            self._name = "%s_%d" % (class_initials, self.identifier)
        return self._name

    @staticmethod
    def _set_name(value):
        del value  # not used
        raise SandboxException("Name is read-only")

    @staticmethod
    def _del_name():
        raise SandboxException("Name is read-only")

    name = property(_get_name, _set_name, _del_name)

    @staticmethod
    def flaten_options(options):
        """
        Convert a list of tuples into space-seperated options+argument string
        """
        result_list = []
        for option, argument in options:
            # positional argument
            if option is None:
                if argument is not None:
                    result_list.append(argument)
                # both empty, ignore
            else:  # option is not None
                # --flag
                if argument is None:
                    result_list.append(option)
                else:  # argument is not None
                    # --option argument or -o argument
                    result_list.append("%s %s" % (option, argument))
        if len(result_list) > 0:
            return " " + " ".join(result_list)
        else:  # they were all (None, None)
            return ""

    def make_sandbox_command_line(self, extra=None):
        """Return entire command-line string needed to start sandbox"""
        command = self.params[self.BINARY_PATH_PARAM]  # mandatory param
        if self.options is not None:
            command += self.flaten_options(self.options)
        if extra is not None:
            command += ' ' + extra
        return command

    def add_optarg(self, option, argument):
        """
        Add an option with an argument into the list of command line options
        """
        if self.options is None:
            self.options = []
        self.options.append((option, argument))

    def add_flag(self, option):
        """
        Add a flag into the list of command line options
        """
        # Tuple encoding required for flaten_options()
        self.add_optarg(option, None)

    def add_pos(self, argument):
        """
        Add a positional option into the list of command line options
        """
        # Tuple encoding required for flaten_options()
        self.add_optarg(None, argument)

    def add_mm(self):
        """
        Append a -- to the end of the current option list
        """
        self.add_pos('--')

    def list_long_options(self):
        """
        Return a list of all long options with an argument
        """
        return [opt for opt, arg in self.options
                if opt.startswith('--') and arg is not None]

    def list_short_options(self):
        """
        Return a list of all short options with an argument
        """
        result = []
        for opt, arg in self.options:
            if arg is None:
                continue  # flag or positional
            if len(opt) > 1 and opt[0] == '-' and opt[1] != '-':
                result.append(opt)

    def list_flags(self):
        """
        Return a list of all flags (options without arguments)
        """
        return [opt for opt, arg in self.options
                if opt.startswith('--') and arg is None]

    def list_pos(self):
        """
        Return a list of all positional arguments
        """
        return [arg for opt, arg in self.options if opt is None]


# Instances are similar to a list-of-lists- multiple kinds (classes) of
# multiple sandobx executions.
class TestSandboxes(object):

    """
    Aggregate manager class of SandboxCommandBase or subclass instances
    """

    # The class of each sandbox instance to operate on
    SANDBOX_TYPE = SandboxCommandBase

    def __init__(self, params, env):
        """
        Create instance(s) of sandbox from a command
        """
        # public attribute for access to each sandbox execution
        self.sandboxes = []
        # Each sandbox type will object_params() itself
        self.params = params
        # In case a subclass wants to interface with tests before/after
        self.env = env
        # Parse out aggregate manager class-specific params
        pop = self.params.object_params(self.__class__.__name__)
        # Allows iterating over all sandboxes e.g. with for_each()
        self.count = int(pop.get('lvsb_count', '1'))
        # Simple-case is all sandboxes on the local host
        self.uri = pop.get('lvsb_uri', 'lxc:///')
        # The command to run inside the sandbox
        self.command = pop.get('lvsb_command')
        # Allows iterating for the options
        self.opts_count = int(pop.get('lvsb_opts_count', '1'))
        # FIXME: should automatically generate this
        self.lvsb_option_mapper = {'optarg': {'connect': '-c', 'name': '-n',
                                              'mount': '-m', 'include': '-i',
                                              'includefile': '-I', 'network': '-N',
                                              'security': '-s'},
                                   'flag': {'help': '-h', 'version': '-V',
                                            'debug': '-d', 'privileged': '-p',
                                            'shell': '-l'}}
        # The list to save options
        self.opts = []
        self.flag = []
        for k in list(self.lvsb_option_mapper.keys()):
            # k may be 'optarg' or 'flag'
            for key, value in list(self.lvsb_option_mapper[k].items()):
                base_name = 'lvsb_%s_options' % key
                for key_gen, option in params.object_counts('lvsb_opts_count',
                                                            base_name):
                    # k is 'optarg'
                    if option and value:
                        self.opts.append((value, option))
                    # k is 'flag'
                    if key_gen in params and not option:
                        self.flag.append(value)

        logging.debug("All of options(%s) and flags(%s)", self.opts, self.flag)

    def init_sandboxes(self):
        """
        Create self.count Sandbox instances
        """
        # self.sandboxes probably empty, can't use for_each()
        for index in xrange(0, self.count):
            del index  # Keep pylint happy
            self.sandboxes.append(self.SANDBOX_TYPE(self.params))

    def for_each(self, do_something, *args, **dargs):
        """
        Iterate over all sandboxes, calling do_something on each

        :param do_sometihng: Called with the item and ``*args``, ``**dargs``
        """
        # Simplify making the same call to every running sandbox
        return [do_something(sandbox, *args, **dargs)
                for sandbox in self.sandboxes]

    def are_running(self):
        """
        Return the number of sandbox processes still running
        """
        running = 0
        for is_running in self.for_each(lambda sb: sb.running()):
            if is_running:
                running += 1
        return running

    def are_failed(self):
        """
        Return the number of sandbox processes with non-zero exit codes
        """
        # Warning, this will block if self.are_running() > 0
        failed = 0
        for exit_code in self.for_each(lambda sb: sb.exit_code()):
            if exit_code != 0:
                failed += 1
        return failed



"""
Base module for support lvm in qemu test;

For EmulatedLVM, no need any special configuration, lvm params will generate
automatically. Of course, customizable params is accept;
For real lvm partition, we need to specify some params, at lest, vg_name and
it's a real volume group on your host. If not, both pv_name and vg_name are
required and a new volumegroup will be created on device named pv_name, But
it will destroy data on your device and it's not recommended;

Required params:
    lv_name:
        lv_name like /dev/vg/lv; If not params["vg_name"]
        is requried and if lv_name not set, use guest_name as
        lv_name; device mapper path (eg, /dev/mapper/vg-lv)
        doesn't support it now;
    lv_size
        string (eg, 30G) if not set image_size will be used;
    vg_name
        LogicalVolume group name, eg, "test_vg";
    pv_name
        PhysicalVolume name eg, /dev/sdb or /dev/sdb1;
"""
from __future__ import division
import os
import time
import re
import math
import logging

from avocado.core import exceptions
from avocado.utils import path
from avocado.utils import process

from virttest import utils_misc
from virttest import data_dir

UNIT = "B"
COMMON_OPTS = "--noheading --nosuffix --unit=%s" % UNIT


def normalize_data_size(size):
    if re.match(".*\d$", str(size)):
        size = "%s%s" % (size, UNIT)
    size = float(utils_misc.normalize_data_size(size, UNIT, 1024))
    return int(math.ceil(size))


def cmd_output(cmd, res="[\w/]+"):
    result = process.run(cmd, ignore_status=True)
    if result.exit_status != 0:
        logging.warn(result)
        return None
    output = result.stdout_text
    for line in output.splitlines():
        val = re.findall(res, line)
        if val:
            return val[0]
    return None


class Volume(object):

    def __init__(self, name, size):
        self.name = name
        self.path = name
        self.size = normalize_data_size(size)

    def get_attr(self, cmd, attr, res="[\w/]+"):
        """
        Get attribue of volume, if not found return None;

        :param cmd: command used to display volume info;
        :param attr: attribue name of the volume;
        :param res: regular expression to reading the attribue;
        :return: string or None
        """
        if attr:
            return cmd_output(cmd, res)
        return None

    def exists(self):
        """
        Check is the volume really exists or not;
        """
        return os.path.exists(self.path)

    def umount(self, extra_args="-f"):
        """
        Unmount volume;
        """
        if self.exists():
            cmd = "umount %s" % extra_args
            fd = open("/proc/mounts", "r")
            mount_list = fd.readlines()
            fd.close()
            for line in mount_list:
                dev, mount_point = line.split()[0], line.split()[2]
                if os.path.exists(dev) and os.path.samefile(dev, self.path):
                    process.system("%s %s" % (cmd, mount_point))


class PhysicalVolume(Volume):

    def __init__(self, name, size):
        super(PhysicalVolume, self).__init__(name, size)
        self.vg = None

    def create(self, extra_args="-ff --yes"):
        """
        Create physical volume on specify physical volume;

        :param extra_args: extra argurments for pvcreate command;
        :raise: CmdError or TestError;
        :return: physical volume abspath
        """
        if not self.exists():
            raise exceptions.TestError("Physical device not found")
        self.umount()
        cmd = "pvcreate %s %s" % (extra_args, self.name)
        process.system(cmd)
        logging.info("Create physical volume: %s", self.name)
        return self.path

    def remove(self, extra_args=" -ff --yes"):
        """
        Remove a physical volume

        :param extra_args: extra argurments for ``pvremove`` command
        :raise: CmdError
        """
        cmd = "lvm pvremove %s %s" % (extra_args, self.name)
        process.system(cmd)
        logging.info("logical physical volume (%s) removed", self.name)

    def resize(self, size, extra_args="-ff --yes"):
        """
        Resize a physical volume;

        :param size: new size of the physical volume device;
        :param extra_args: extra argurments for pvresize command;
        """
        size = int(math.ceil(normalize_data_size(size)))
        cmd = "lvm pvresize %s --setphysicalvolumesize=%s%s %s" % (extra_args,
                                                                   size,
                                                                   UNIT,
                                                                   self.name)
        process.system(cmd)
        self.size = size
        logging.info("resize volume %s to %s B" % (self.name, self.size))

    def display(self):
        """
        Show physical volume detials

        :raise: CmdError
        """
        cmd = "pvdisplay %s" % self.name
        process.system(cmd)

    def get_attr(self, attr):
        """
        Get attribue of physical volume, if not found return None;

        :param attr: attribue name of the volume;
        :return: string or None
        """
        cmd = "lvm pvs -o %s %s %s" % (attr, COMMON_OPTS, self.name)
        return super(PhysicalVolume, self).get_attr(cmd, attr)

    def set_vg(self, vg):
        """
        Set VolumeGroup of the physical volume device;

        :param vg: VolumeGroup object
        """
        if isinstance(vg, VolumeGroup):
            self.vg = vg


class VolumeGroup(object):

    def __init__(self, name, size, pvs):
        self.name = name
        self.size = normalize_data_size(size)
        self.pvs = pvs
        self.lvs = []

    def create(self, extra_args="-ff --yes"):
        """
        Create volume group with specify physical volumes;

        :param extra_args: extra argurments for lvm command;
        :raise: CmdError or TestError;
        :return: volume group name;
        """
        cmd = "lvm vgcreate  %s %s" % (extra_args, self.name)
        for pv in self.pvs:
            if pv.vg and pv.vg.name != self.name:
                try:
                    pv.vg.reduce_pv(pv)
                except Exception:
                    pv.vg.remove()
            cmd += " %s" % pv.name
        process.system(cmd)
        logging.info("Create new volumegroup %s", self.name)
        return self.name

    def remove(self, extra_args="-ff --yes"):
        """
        Remove the VolumeGroup;

        :param extra_args: extra argurments for lvm command;
        """
        cmd = "lvm vgremove %s %s" % (extra_args, self.name)
        process.system(cmd)
        logging.info("logical volume-group(%s) removed", self.name)

    def get_attr(self, attr):
        """
        Get VolumeGroup attribue;

        :param attr: attribue name;
        :return: string or None;
        """
        cmd = "lvm vgs -o %s %s %s" % (attr, COMMON_OPTS, self.name)
        return cmd_output(cmd)

    def append_lv(self, lv):
        """
        Collect Logical Volumes on the VolumeGroup;

        :param lv: LogicalVolume Object
        """
        if isinstance(lv, LogicalVolume):
            if lv not in self.lvs:
                self.lvs.append(lv)

    def reduce_pv(self, pv, extra_args="-ff --yes"):
        """
        Reduce a PhysicalVolume from VolumeGroup;

        :param pv: PhysicalVolume object;
        :param extra_args: extra argurments pass to lvm command;
        """
        if not isinstance(pv, PhysicalVolume):
            raise TypeError("Need a PhysicalVolume object")
        cmd = "lvm vgreduce %s %s %s" % (extra_args, self.name, pv.name)
        process.system(cmd)
        self.pvs.remove(pv)
        logging.info("reduce volume %s from volume group %s" % (pv.name,
                                                                self.name))

    def extend_pv(self, pv, extra_args=""):
        """
        Add PhysicalVolume into VolumeGroup;

        :param pv: PhysicalVolume object
        :param extra_args: extra argurments used for vgextend command
        """
        if not isinstance(pv, PhysicalVolume):
            raise TypeError("Need a PhysicalVolume object")
        cmd = "lvm vgextend %s %s" % (self.name, pv.name)
        process.system(cmd)
        self.pvs.append(pv)
        logging.info("add volume %s to volumegroup %s" % (pv.name, self.name))

    def exists(self):
        """
        Check VolumeGroup exists or not;

        :return: bool type, if exists True else False;
        """
        vg_name = self.get_attr("vg_name")
        return bool(vg_name)


class LogicalVolume(Volume):

    def __init__(self, name, size, vg):
        super(LogicalVolume, self).__init__(name, size)
        self.vg = vg
        self.path = os.path.join("/dev", vg.name, name)

    def create(self):
        """
        Create LogicalVolume device;

        :return: path of logical volume;
        """
        vg_name = self.vg.name
        cmd = "lvm lvcreate -L %s%s -n %s %s" % (self.size,
                                                 UNIT,
                                                 self.name,
                                                 vg_name)
        process.system(cmd)
        logging.info("create logical volume %s", self.path)
        return self.get_attr("lv_path")

    def remove(self, extra_args="-ff --yes", timeout=300):
        """
        Remove LogicalVolume device;

        :param extra_args: extra argurments pass to lvm command;
        :param timeout: timeout in seconds;
        """
        end_time = time.time() + timeout
        while time.time() < end_time:
            self.umount()
            cmd = "lvm lvremove %s %s/%s" % (extra_args,
                                             self.vg.name, self.name)
            status = process.system(cmd, ignore_status=True)
            if status == 0:
                logging.info("logical volume(%s) removed", self.name)
                break
            time.sleep(0.5)

    def resize(self, size, extra_args="-ff"):
        """
        Resize LogicalVolume to new size;

        :param size: new size of logical volume;
        :param extra_args: extra argurments pass to lvm command;
        :return: size of logical volume;
        """
        path = self.get_attr("lv_path")
        size = str(size)
        if size.startswith("+"):
            size = self.size + normalize_data_size(size[1:])
        elif size.startswith("-"):
            size = self.size - normalize_data_size(size[1:])
        else:
            size = normalize_data_size(size)
        cmd = "lvm lvresize -n -L %s%s %s %s" % (size, UNIT, path, extra_args)
        process.system(cmd)
        self.size = size
        logging.info("resize logical volume %s size to %s" % (self.path,
                                                              self.size))
        return size

    def display(self, extra_args=""):
        """
        Shown logical volume detials, warper of lvm command lvdisplay;

        :extra_args: extra argurments pass to lvdisplay command;
        :raise: CmdError when command exit code not equal 0;
        """
        path = self.get_attr("lv_path")
        cmd = "lvm lvs %s %s" % (extra_args, path)
        return process.system(cmd)

    def get_attr(self, attr):
        """
        Get logical volume attribues if not found return None;

        :param attr: attribue name;
        :return: attribue value string or None;
        :raise: CmdError when command exit code not equal 0;
        """
        cmd = "lvm lvs -o %s %s %s" % (attr, COMMON_OPTS, self.path)
        return super(LogicalVolume, self).get_attr(cmd, attr)


class LVM(object):

    def __init__(self, params):
        path.find_command("lvm")
        self.params = self.__format_params(params)
        self.pvs = self.__reload_pvs()
        self.vgs = self.__reload_vgs()
        self.lvs = self.__reload_lvs()
        self.trash = []

    def generate_id(self, params):
        """
        Create prefix with image_name;
        """
        black_str = re.compile(r"[-./]")
        return black_str.sub("_", os.path.basename(params["image_name"]))

    def __format_params(self, params):
        """
        Reformat test params;

        :param params: dict of test params;
        :return: dict of test params;
        """
        lv_size = params.get("lv_size")
        if lv_size is None:
            lv_size = params["image_size"]
        params["lv_size"] = normalize_data_size(lv_size)

        lv_name = params.get("lv_name")
        if lv_name is None:
            lv_name = "lv_%s" % self.generate_id(params)
            params["lv_name"] = lv_name

        vg_name = params.get("vg_name")
        if vg_name is None:
            vg_name = "vg_%s" % self.generate_id(params)
            params["vg_name"] = vg_name
        if lv_name.startswith("/dev"):
            if "mapper" not in lv_name:
                match = re.search("/dev/([\w_]+)/([\w_]+)", lv_name)
                vg_name, lv_name = [x[1:] for x in match.groups()]
                params["lv_name"] = lv_name
                params["vg_name"] = vg_name
        return params

    def register(self, vol):
        """
        Register new volume;

        :param vol: Volume object or VolumeGroup objects
        """
        if isinstance(vol, Volume) or isinstance(vol, VolumeGroup):
            self.trash.append(vol)
            logging.info("Install new volume %s", vol.name)

    def unregister(self, vol):
        """
        Unregister volume or VolumeGroup;

        :param vol: Volume object or VolumeGroup objects
        """
        if vol in self.trash:
            self.trash.remove(vol)
            logging.info("Uninstall volume %s", vol.name)

    def __reload_lvs(self):
        """
        Create LogicalVolume objects for exist Logical volumes;

        :return: list of Volume object
        """
        lvs = []
        cmd = "lvm lvs -o lv_name,lv_size,vg_name %s" % COMMON_OPTS
        output = process.run(cmd).stdout_text
        for line in output.splitlines():
            lv_name, lv_size, vg_name = line.split()
            vg = self.get_vol(vg_name, "vgs")
            lv = LogicalVolume(lv_name, lv_size, vg)
            vg.append_lv(lv)
            lvs.append(lv)
        return lvs

    def __reload_vgs(self):
        """
        Create VolumeGroup objects for exist volumegroups;

        :return: list of Volume object
        """
        vgs = []
        cmd = "lvm vgs -opv_name,vg_name,vg_size %s" % COMMON_OPTS
        output = process.run(cmd).stdout_text
        for line in output.splitlines():
            pv_name, vg_name, vg_size = line.split()
            pv = self.get_vol(pv_name, "pvs")
            vg = VolumeGroup(vg_name, vg_size, [pv])
            pv.set_vg(vg)
            vgs.append(vg)
        return vgs

    def __reload_pvs(self):
        """
        Create PhysicalVolume objects for exist physical volumes;

        :return: list of Volume object
        """
        pvs = []
        cmd = "lvm pvs -opv_name,pv_size %s" % COMMON_OPTS
        output = process.run(cmd).stdout_text
        for line in output.splitlines():
            pv_name, pv_size = line.split()
            pv = PhysicalVolume(pv_name, pv_size)
            pvs.append(pv)
        return pvs

    def get_vol(self, vname, vtype):
        """
        Get a exists volume object;

        :param vname: volume name;
        :param vtype: volume type eg, 'pvs', 'vgs', 'lvs';
        :return: Volume object or None;
        """
        if vtype:
            vols = getattr(self, vtype)
            for vol in vols:
                if vol.name == vname:
                    return vol
        return None

    def setup_pv(self, vg):
        """
        Create a physical volume devices;

        :param params["pv_name"]: Physical volume devices path or mount point;
        :param vg: VolumeGroup object;
        :return: list of PhysicalVolume object;
        """
        pvs = []
        for pv_name in self.params["pv_name"].split():
            pv = self.get_vol(pv_name, "pvs")
            if pv is None:
                pv = PhysicalVolume(pv_name, 0)
                pv.create()
                self.register(pv)
                self.pvs.append(pv)
            pv.set_vg(vg)
            pvs.append(pv)
        return pvs

    def setup_vg(self, lv):
        """
        Setup logical volumegroup which specify on volumegroup specify by
        params["vg_name"];

        :param params["vg_name"]: volumegroup name;
        :return: volumegroup object;
        """
        vg_name = self.params["vg_name"]
        vg = self.get_vol(vg_name, "vgs")
        if vg is None:
            pvs = self.setup_pv(vg)
            vg = VolumeGroup(vg_name, 0, pvs)
            vg.create()
            self.register(vg)
            for pv in pvs:
                pv.set_vg(vg)
            self.vgs.append(vg)
        else:
            logging.info("VolumeGroup(%s) really exists" % vg_name +
                         "skip to create it")
            pv_name = self.params["pv_name"].split()[0]
            pv = self.get_vol(pv_name, "pvs")
            if pv and pv.vg is vg:
                vg.append_lv(lv)
                return vg
            # if set pv_name then add pvs into volume group
            pvs = self.setup_pv(vg)
            for pv in pvs:
                vg.extend_pv(pv)
        vg.append_lv(lv)
        return vg

    def setup_lv(self):
        """
        Setup a logical volume, if a exist logical volume resize it
        else then create it on specify volumegroup;

        :param params["lv_name"]: logical volume name;
        :param params["lv_name"]: logical volume size;
        :return: logical volume object;
        """
        lv_name = self.params["lv_name"]
        lv_size = self.params["lv_size"]
        lv = self.get_vol(lv_name, "lvs")
        # Check is it a exist lv if exist return the volume object
        # else then create it;
        if lv is None:
            vg = self.setup_vg(lv)
            lv = LogicalVolume(lv_name, lv_size, vg)
            lv.create()
            self.register(lv)
            self.lvs.append(lv)
        else:
            logging.info("LogicalVolume(%s) really exists " % lv_name +
                         "skip to create it")
        if lv.size != lv_size:
            lv.display()
            logging.warn("lv size(%s) mismath," % lv.size +
                         "requried size %s;" % lv_size)
            lv.resize(lv_size)
        return lv

    def setup(self):
        """
        Main function to setup a lvm environments;

        :return: LogicalVolume path
        """
        self.rescan()
        lv = self.setup_lv()
        return lv.get_attr("lv_path")

    def cleanup(self):
        """
        Remove useless lv, vg and pv then reload lvm releated service;
        """
        if self.params.get("force_remove_image", "no") == "yes":
            self.trash.reverse()
            trash = self.trash[:]
            for vol in trash:
                if isinstance(vol, LogicalVolume):
                    vol.umount()
                if isinstance(vol, PhysicalVolume):
                    vg = vol.vg
                    if vg is not None:
                        vg.reduce_pv(vol)
                if isinstance(vol, VolumeGroup):
                    for pv in self.pvs:
                        if pv.vg is vol:
                            pv.vg = None
                vol.remove()
                self.unregister(vol)
        self.rescan()

    def rescan(self):
        """
        Rescan lvm , used before create volume or after remove volumes;
        """
        lvm_reload_cmd = self.params.get("lvm_reload_cmd")
        if lvm_reload_cmd:
            process.system(lvm_reload_cmd, ignore_status=True)
            logging.info("reload lvm monitor service")


class EmulatedLVM(LVM):

    def __init__(self, params, root_dir=data_dir.get_tmp_dir()):
        path.find_command("losetup")
        path.find_command("dd")
        super(EmulatedLVM, self).__init__(params)
        self.data_dir = root_dir

    def get_emulate_image_name(self):
        img_path = self.params.get("emulated_image")
        if img_path is None:
            img_path = self.generate_id(self.params)
        return utils_misc.get_path(self.data_dir, img_path)

    def make_emulate_image(self):
        """
        Create emulate image via dd with 8M block size;
        """
        img_size = self.params["lv_size"]
        img_path = self.get_emulate_image_name()
        bs_size = normalize_data_size("8M")
        count = int(math.ceil(img_size / bs_size)) + 8
        logging.info("create emulated image file(%s)" % img_path)
        cmd = "dd if=/dev/zero of=%s bs=8M count=%s" % (img_path, count)
        process.system(cmd)
        self.params["pv_size"] = count * bs_size
        return img_path

    def make_volume(self, img_file, extra_args=""):
        """
        Map a file to loop back device;

        :param img_file: image file path;
        :return: loop back device name;
        """
        cmd = "losetup %s --show --find %s" % (extra_args, img_file)
        pv_name = process.run(cmd).stdout_text
        self.params["pv_name"] = pv_name.strip()
        return pv_name

    def setup_pv(self, vg):
        """
        Setup physical volume device if exists return it directly;
        """
        pvs = []
        emulate_image_file = self.get_emulate_image_name()
        cmd = "losetup -j %s" % emulate_image_file
        output = process.run(cmd).stdout_text
        try:
            pv_name = re.findall("(/dev/loop\d+)", output, re.M | re.I)[-1]
            pv = self.get_vol(pv_name, "pvs")
        except IndexError:
            pv = None
        if pv is None:
            img_file = self.make_emulate_image()
            pv_name = self.make_volume(img_file)
            pv_size = self.params["pv_size"]
            pv = PhysicalVolume(pv_name, pv_size)
            pv.create()
            self.register(pv)
        else:
            logging.warn("PhysicalVolume(%s) really exists" % pv_name +
                         "skip to create it")
        pv.set_vg(vg)
        pvs.append(pv)
        return pvs

    def setup(self):
        """
        Main function to setup a lvm environments;

        :return: LogicalVolume path
        """
        self.rescan()
        lv = self.setup_lv()
        if "/dev/loop" not in lv.get_attr("devices"):
            lv.display()
            raise exceptions.TestError("logical volume exists but is not a " +
                                       "emulated logical device")
        return lv.get_attr("lv_path")

    def cleanup(self):
        """
        Cleanup created logical volumes;
        """
        super(EmulatedLVM, self).cleanup()
        if self.params.get("remove_emulated_image", "no") == "yes":
            emulate_image_file = self.get_emulate_image_name()
            cmd = "losetup -j %s" % emulate_image_file
            output = process.run(cmd).stdout_text
            devices = re.findall("(/dev/loop\d+)", output, re.M | re.I)
            for dev in devices:
                cmd = "losetup -d %s" % dev
                logging.info("disconnect %s", dev)
                process.system(cmd, ignore_status=True)
            emulate_image_file = self.get_emulate_image_name()
            cmd = "rm -f %s" % emulate_image_file
            process.system(cmd, ignore_status=True)
            logging.info("remove emulate image file %s", emulate_image_file)



import platform

from avocado.utils import cpu

ARCH = platform.machine()

if ARCH in ('ppc64', 'ppc64le'):
    # From include/linux/sockios.h
    SIOCSIFHWADDR = 0x8924
    SIOCGIFHWADDR = 0x8927
    SIOCGIFFLAGS = 0x8913
    SIOCSIFFLAGS = 0x8914
    SIOCGIFADDR = 0x8915
    SIOCSIFADDR = 0x8916
    SIOCGIFNETMASK = 0x891B
    SIOCSIFNETMASK = 0x891C
    SIOCGIFMTU = 0x8921
    SIOCSIFMTU = 0x8922
    SIOCGIFINDEX = 0x8933
    SIOCBRADDIF = 0x89a2
    SIOCBRDELIF = 0x89a3
    SIOCBRADDBR = 0x89a0
    SIOCBRDELBR = 0x89a1
    # From linux/include/linux/if_tun.h
    TUNSETIFF = 0x800454ca
    TUNGETIFF = 0x400454d2
    TUNGETFEATURES = 0x400454cf
    TUNSETQUEUE = 0x800454d9
    IFF_MULTI_QUEUE = 0x0100
    IFF_TAP = 0x2
    IFF_NO_PI = 0x1000
    IFF_VNET_HDR = 0x4000
    # From linux/include/linux/if.h
    IFF_UP = 0x1
    IFF_PROMISC = 0x100
    # From linux/netlink.h
    NETLINK_ROUTE = 0
    NLM_F_REQUEST = 1
    NLM_F_ACK = 4
    RTM_DELLINK = 17
    NLMSG_ERROR = 2
    # From linux/socket.h
    AF_PACKET = 17
    # From linux/vhost.h
    VHOST_VSOCK_SET_GUEST_CID = 0x8008af60
else:
    # From include/linux/sockios.h
    SIOCSIFHWADDR = 0x8924
    SIOCGIFHWADDR = 0x8927
    SIOCGIFFLAGS = 0x8913
    SIOCSIFFLAGS = 0x8914
    SIOCGIFADDR = 0x8915
    SIOCSIFADDR = 0x8916
    SIOCGIFNETMASK = 0x891B
    SIOCSIFNETMASK = 0x891C
    SIOCGIFMTU = 0x8921
    SIOCSIFMTU = 0x8922
    SIOCGIFINDEX = 0x8933
    SIOCBRADDIF = 0x89a2
    SIOCBRDELIF = 0x89a3
    SIOCBRADDBR = 0x89a0
    SIOCBRDELBR = 0x89a1
    # From linux/include/linux/if_tun.h
    TUNSETIFF = 0x400454ca
    TUNGETIFF = 0x800454d2
    TUNGETFEATURES = 0x800454cf
    TUNSETQUEUE = 0x400454d9
    IFF_MULTI_QUEUE = 0x0100
    IFF_TAP = 0x0002
    IFF_NO_PI = 0x1000
    IFF_VNET_HDR = 0x4000
    # From linux/include/linux/if.h
    IFF_UP = 0x1
    IFF_PROMISC = 0x100
    # From linux/netlink.h
    NETLINK_ROUTE = 0
    NLM_F_REQUEST = 1
    NLM_F_ACK = 4
    RTM_DELLINK = 17
    NLMSG_ERROR = 2
    # From linux/socket.h
    AF_PACKET = 17
    # From linux/vhost.h
    VHOST_VSOCK_SET_GUEST_CID = 0x4008af60


def get_kvm_module_list():
    if ARCH == 'x86_64':
        vendor = cpu.get_vendor() if hasattr(cpu, 'get_vendor') else cpu.get_cpu_vendor_name()
        return ["kvm", "kvm-%s" % vendor]
    elif ARCH in ('ppc64', 'ppc64le'):
        # FIXME: Please correct it if anyone still want to use KVM-PR mode
        return ["kvm", "kvm-hv"]
    elif ARCH in ('s390', 's390x'):
        return ["kvm"]
    elif ARCH == "aarch64":
        return []



# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# See LICENSE for more details.
#
# Copyright: Red Hat Inc. 2015
# Author: Lucas Meneghel Rodrigues <lmr@redhat.com>

"""
Avocado VT plugin
"""

import logging
import os

from avocado.utils import path as utils_path

from virttest import cartesian_config
from virttest import data_dir
from virttest import defaults
from virttest import standalone_test
from virttest.compat import get_opt, set_opt, set_opt_from_settings
from virttest.standalone_test import SUPPORTED_DISK_BUSES
from virttest.standalone_test import SUPPORTED_IMAGE_TYPES
from virttest.standalone_test import SUPPORTED_LIBVIRT_DRIVERS
from virttest.standalone_test import SUPPORTED_NET_TYPES
from virttest.standalone_test import SUPPORTED_NIC_MODELS
from virttest.standalone_test import SUPPORTED_TEST_TYPES


class VirtTestOptionsProcess(object):

    """
    Pick virt test options and parse them to get to a cartesian parser.
    """

    def __init__(self, config):
        """
        Parses options and initializes attributes.
        """
        self.config = config
        # Here we'll inject values from the config file.
        # Doing this makes things configurable yet the number of options
        # is not overwhelming.
        # setup section
        set_opt_from_settings(self.config,
                              'vt.setup', 'backup_image_before_test',
                              key_type=bool, default=True)
        set_opt_from_settings(self.config,
                              'vt.setup', 'restore_image_after_test',
                              key_type=bool, default=True)
        set_opt_from_settings(self.config,
                              'vt.setup', 'keep_guest_running',
                              key_type=bool, default=False)
        # common section
        set_opt_from_settings(self.config,
                              'vt.common', 'data_dir',
                              default=None)
        set_opt_from_settings(self.config,
                              'vt.common', 'tmp_dir',
                              default='')
        set_opt_from_settings(self.config,
                              'vt.common', 'type_specific',
                              key_type=bool, default=False)
        set_opt_from_settings(self.config,
                              'vt.common', 'mem',
                              default=None)
        set_opt_from_settings(self.config,
                              'vt.common', 'nettype',
                              default=None)
        set_opt_from_settings(self.config,
                              'vt.common', 'netdst',
                              default='virbr0')
        # qemu section
        set_opt_from_settings(self.config,
                              'vt.qemu', 'accel',
                              default='kvm')
        set_opt_from_settings(self.config,
                              'vt.qemu', 'vhost',
                              default='off')
        set_opt_from_settings(self.config,
                              'vt.qemu', 'monitor',
                              default=None)
        set_opt_from_settings(self.config,
                              'vt.qemu', 'smp',
                              default='2')
        set_opt_from_settings(self.config,
                              'vt.qemu', 'image_type',
                              default=SUPPORTED_IMAGE_TYPES[0])
        set_opt_from_settings(self.config,
                              'vt.qemu', 'nic_model',
                              default=SUPPORTED_NIC_MODELS[0])
        set_opt_from_settings(self.config,
                              'vt.qemu', 'disk_bus',
                              default=SUPPORTED_DISK_BUSES[0])
        set_opt_from_settings(self.config,
                              'vt.qemu', 'sandbox',
                              default='on')
        set_opt_from_settings(self.config,
                              'vt.qemu', 'defconfig',
                              default='yes')
        set_opt_from_settings(self.config,
                              'vt.qemu', 'malloc_perturb',
                              default='yes')

        # debug section
        set_opt_from_settings(self.config,
                              'vt.debug', 'no_cleanup',
                              key_type=bool, default=False)

        self.cartesian_parser = None

    def _process_qemu_bin(self):
        """
        Puts the value of the qemu bin option in the cartesian parser command.
        """
        qemu_bin_setting = ('option --vt-qemu-bin or '
                            'config vt.qemu.qemu_bin')
        if (get_opt(self.config, 'vt.config') and
                get_opt(self.config, 'vt.qemu.qemu_bin') is None):
            logging.info("Config provided and no %s set. Not trying "
                         "to automatically set qemu bin.", qemu_bin_setting)
        else:
            (qemu_bin_path, qemu_img_path, qemu_io_path,
             qemu_dst_bin_path) = standalone_test.find_default_qemu_paths(
                 get_opt(self.config, 'vt.qemu.qemu_bin'),
                 get_opt(self.config, 'vt.qemu.qemu_dst_bin'))
            self.cartesian_parser.assign("qemu_binary", qemu_bin_path)
            self.cartesian_parser.assign("qemu_img_binary", qemu_img_path)
            self.cartesian_parser.assign("qemu_io_binary", qemu_io_path)
            if qemu_dst_bin_path is not None:
                self.cartesian_parser.assign("qemu_dst_binary",
                                             qemu_dst_bin_path)

    def _process_qemu_img(self):
        """
        Puts the value of the qemu bin option in the cartesian parser command.
        """
        qemu_img_setting = ('option --vt-qemu-img or '
                            'config vt.qemu.qemu_img')
        if (get_opt(self.config, 'vt.config') and
                get_opt(self.config, 'vt.qemu.bin') is None):
            logging.info("Config provided and no %s set. Not trying "
                         "to automatically set qemu bin", qemu_img_setting)
        else:
            (_, qemu_img_path,
             _, _) = standalone_test.find_default_qemu_paths(
                 get_opt(self.config, 'vt.qemu.qemu_bin'),
                 get_opt(self.config, 'vt.qemu.qemu_dst_bin'))
            self.cartesian_parser.assign("qemu_img_binary", qemu_img_path)

    def _process_qemu_accel(self):
        """
        Puts the value of the qemu bin option in the cartesian parser command.
        """
        if get_opt(self.config, 'vt.qemu.accel') == 'tcg':
            self.cartesian_parser.assign("disable_kvm", "yes")

    def _process_bridge_mode(self):
        nettype_setting = 'config vt.qemu.nettype'
        if not get_opt(self.config, 'vt.config'):
            # Let's select reasonable defaults depending on vt.type
            if not get_opt(self.config, 'vt.common.nettype'):
                if get_opt(self.config, 'vt.type') == 'qemu':
                    set_opt(self.config, 'vt.common.nettype',
                            ("bridge" if os.getuid() == 0 else "user"))
                elif get_opt(self.config, 'vt.type') == 'spice':
                    set_opt(self.config, 'vt.common.nettype', "none")
                else:
                    set_opt(self.config, 'vt.common.nettype', "bridge")

            if get_opt(self.config, 'vt.common.nettype') not in SUPPORTED_NET_TYPES:
                raise ValueError("Invalid %s '%s'. "
                                 "Valid values: (%s)" %
                                 (nettype_setting,
                                  get_opt(self.config, 'vt.common.nettype'),
                                  ", ".join(SUPPORTED_NET_TYPES)))
            if get_opt(self.config, 'vt.common.nettype') == 'bridge':
                if os.getuid() != 0:
                    raise ValueError("In order to use %s '%s' you "
                                     "need to be root" % (nettype_setting,
                                                          get_opt(self.config, 'vt.common.nettype')))
                self.cartesian_parser.assign("nettype", "bridge")
                self.cartesian_parser.assign("netdst", get_opt(self.config, 'vt.common.netdst'))
            elif get_opt(self.config, 'vt.common.nettype') == 'user':
                self.cartesian_parser.assign("nettype", "user")
        else:
            logging.info("Config provided, ignoring %s", nettype_setting)

    def _process_monitor(self):
        if not get_opt(self.config, 'vt.config'):
            if not get_opt(self.config, 'vt.qemu.monitor'):
                pass
            elif get_opt(self.config, 'vt.qemu.monitor') == 'qmp':
                self.cartesian_parser.assign("monitor_type", "qmp")
            elif get_opt(self.config, 'vt.qemu.monitor') == 'human':
                self.cartesian_parser.assign("monitor_type", "human")
        else:
            logging.info("Config provided, ignoring monitor setting")

    def _process_smp(self):
        smp_setting = 'config vt.qemu.smp'
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.qemu.smp') == '1':
                self.cartesian_parser.only_filter("up")
            elif get_opt(self.config, 'vt.qemu.smp') == '2':
                self.cartesian_parser.only_filter("smp2")
            else:
                try:
                    self.cartesian_parser.only_filter("up")
                    self.cartesian_parser.assign(
                        "smp", int(get_opt(self.config, 'vt.qemu.smp')))
                except ValueError:
                    raise ValueError("Invalid %s '%s'. Valid value: (1, 2, "
                                     "or integer)" % get_opt(self.config, 'vt.qemu.smp'))
        else:
            logging.info("Config provided, ignoring %s", smp_setting)

    def _process_arch(self):
        if get_opt(self.config, 'vt.config'):
            arch_setting = "option --vt-arch or config vt.common.arch"
            logging.info("Config provided, ignoring %s", arch_setting)
            return

        arch = get_opt(self.config, 'vt.common.arch')
        if arch:
            self.cartesian_parser.only_filter(arch)

    def _process_machine_type(self):
        machine_type_setting = ("option --vt-machine-type or config "
                                "vt.common.machine_type")
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.common.machine_type') is None:
                # TODO: this is x86-specific, instead we can get the default
                # arch from qemu binary and run on all supported machine types
                if ((get_opt(self.config, 'vt.common.arch') is None) and
                        (get_opt(self.config, 'vt.guest_os') is None)):
                    self.cartesian_parser.only_filter(
                        defaults.DEFAULT_MACHINE_TYPE)
            else:
                self.cartesian_parser.only_filter(get_opt(self.config,
                                                          'vt.common.machine_type'))
        else:
            logging.info("Config provided, ignoring %s", machine_type_setting)

    def _process_image_type(self):
        image_type_setting = 'config vt.qemu.image_type'
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.qemu.image_type') in SUPPORTED_IMAGE_TYPES:
                self.cartesian_parser.only_filter(get_opt(self.config, 'vt.qemu.image_type'))
            else:
                self.cartesian_parser.only_filter("raw")
                # The actual param name is image_format.
                self.cartesian_parser.assign("image_format",
                                             get_opt(self.config, 'vt.qemu.image_type'))
        else:
            logging.info("Config provided, ignoring %s", image_type_setting)

    def _process_nic_model(self):
        nic_model_setting = 'config vt.qemu.nic_model'
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.qemu.nic_model') in SUPPORTED_NIC_MODELS:
                self.cartesian_parser.only_filter(get_opt(self.config, 'vt.qemu.nic_model'))
            else:
                self.cartesian_parser.only_filter("nic_custom")
                self.cartesian_parser.assign(
                    "nic_model", get_opt(self.config, 'vt.qemu.nic_model'))
        else:
            logging.info("Config provided, ignoring %s", nic_model_setting)

    def _process_disk_buses(self):
        disk_bus_setting = 'config vt.qemu.disk_bus'
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.qemu.disk_bus') in SUPPORTED_DISK_BUSES:
                self.cartesian_parser.only_filter(get_opt(self.config, 'vt.qemu.disk_bus'))
            else:
                raise ValueError("Invalid %s '%s'. Valid values: %s" %
                                 (disk_bus_setting,
                                  get_opt(self.config, 'vt.qemu.disk_bus'),
                                  SUPPORTED_DISK_BUSES))
        else:
            logging.info("Config provided, ignoring %s", disk_bus_setting)

    def _process_vhost(self):
        nettype_setting = 'config vt.qemu.nettype'
        vhost_setting = 'config vt.qemu.vhost'
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.common.nettype') == "bridge":
                if get_opt(self.config, 'vt.qemu.vhost') == "on":
                    self.cartesian_parser.assign("vhost", "on")
                elif get_opt(self.config, 'vt.qemu.vhost') == "force":
                    self.cartesian_parser.assign("netdev_extra_params",
                                                 '",vhostforce=on"')
                    self.cartesian_parser.assign("vhost", "on")
            else:
                if get_opt(self.config, 'vt.qemu.vhost') in ["on", "force"]:
                    raise ValueError("%s '%s' is incompatible with %s '%s'"
                                     % (nettype_setting,
                                        get_opt(self.config, 'vt.common.nettype'),
                                        vhost_setting,
                                        get_opt(self.config, 'vt.qemu.vhost')))
        else:
            logging.info("Config provided, ignoring %s", vhost_setting)

    def _process_qemu_sandbox(self):
        sandbox_setting = 'config vt.qemu.sandbox'
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.qemu.sandbox') == "off":
                self.cartesian_parser.assign("qemu_sandbox", "off")
        else:
            logging.info("Config provided, ignoring %s", sandbox_setting)

    def _process_qemu_defconfig(self):
        defconfig_setting = 'config vt.qemu.sandbox'
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.qemu.defconfig') == "no":
                self.cartesian_parser.assign("defconfig", "no")
        else:
            logging.info("Config provided, ignoring %s", defconfig_setting)

    def _process_malloc_perturb(self):
        self.cartesian_parser.assign("malloc_perturb",
                                     get_opt(self.config, 'vt.qemu.malloc_perturb'))

    def _process_qemu_specific_options(self):
        """
        Calls for processing all options specific to the qemu test.

        This method modifies the cartesian set by parsing additional lines.
        """

        self._process_qemu_bin()
        self._process_qemu_accel()
        self._process_monitor()
        self._process_smp()
        self._process_image_type()
        self._process_nic_model()
        self._process_disk_buses()
        self._process_vhost()
        self._process_malloc_perturb()
        self._process_qemu_sandbox()

    def _process_lvsb_specific_options(self):
        """
        Calls for processing all options specific to LVSB test
        """
        set_opt(self.config, 'no_downloads', True)

    def _process_libvirt_specific_options(self):
        """
        Calls for processing all options specific to libvirt test.
        """
        uri_setting = 'config vt.libvirt.connect_uri'
        if get_opt(self.config, 'vt.libvirt.connect_uri'):
            driver_found = False
            for driver in SUPPORTED_LIBVIRT_DRIVERS:
                if get_opt(self.config, 'vt.libvirt.connect_uri').count(driver):
                    driver_found = True
                    self.cartesian_parser.only_filter(driver)
            if not driver_found:
                raise ValueError("Unsupported %s '%s'"
                                 % (uri_setting,
                                    get_opt(self.config,
                                            'vt.libvbirt.connect_uri')))
        else:
            self.cartesian_parser.only_filter("qemu")

    def _process_guest_os(self):
        guest_os_setting = 'option --vt-guest-os'

        if get_opt(self.config, 'vt.type') == 'spice':
            logging.info("Ignoring predefined OS: %s", guest_os_setting)
            return

        if not get_opt(self.config, 'vt.config'):
            if len(standalone_test.get_guest_name_list(self.config)) == 0:
                raise ValueError("%s '%s' is not on the known guest os for "
                                 "arch '%s' and machine type '%s'. (see "
                                 "--vt-list-guests)"
                                 % (guest_os_setting,
                                    get_opt(self.config, 'vt.guest_os'),
                                    get_opt(self.config, 'vt.common.arch'),
                                    get_opt(self.config, 'vt.common.machine_type')))
            self.cartesian_parser.only_filter(
                get_opt(self.config, 'vt.guest_os') or defaults.DEFAULT_GUEST_OS)
        else:
            logging.info("Config provided, ignoring %s", guest_os_setting)

    def _process_restart_vm(self):
        if not get_opt(self.config, 'vt.config'):
            if not get_opt(self.config, 'vt.setup.keep_guest_running'):
                self.cartesian_parser.assign("kill_vm", "yes")

    def _process_restore_image(self):
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.setup.backup_image_before_test'):
                self.cartesian_parser.assign("backup_image_before_testing",
                                             "yes")
            if get_opt(self.config, 'vt.setup.restore_image_after_test'):
                self.cartesian_parser.assign("restore_image_after_testing",
                                             "yes")

    def _process_mem(self):
        if not get_opt(self.config, 'vt.config'):
            mem = get_opt(self.config, 'vt.common.mem')
            if mem is not None:
                self.cartesian_parser.assign("mem", mem)

    def _process_tcpdump(self):
        """
        Verify whether we can run tcpdump. If we can't, turn it off.
        """
        try:
            tcpdump_path = utils_path.find_command('tcpdump')
        except utils_path.CmdNotFoundError:
            tcpdump_path = None

        non_root = os.getuid() != 0

        if tcpdump_path is None or non_root:
            self.cartesian_parser.assign("run_tcpdump", "no")

    def _process_no_filter(self):
        if get_opt(self.config, 'vt.no_filter'):
            for item in get_opt(self.config, 'vt.no_filter').split(' '):
                self.cartesian_parser.no_filter(item)

    def _process_only_filter(self):
        if get_opt(self.config, 'vt.only_filter'):
            for item in get_opt(self.config, 'vt.only_filter').split(' '):
                self.cartesian_parser.only_filter(item)

    def _process_extra_params(self):
        if get_opt(self.config, "vt.extra_params"):
            for param in get_opt(self.config, "vt.extra_params"):
                key, value = param.split('=', 1)
                self.cartesian_parser.assign(key, value)

    def _process_only_type_specific(self):
        if not get_opt(self.config, 'vt.config'):
            if get_opt(self.config, 'vt.type_specific'):
                self.cartesian_parser.only_filter("(subtest=type_specific)")

    def _process_general_options(self):
        """
        Calls for processing all generic options.

        This method modifies the cartesian set by parsing additional lines.
        """
        self._process_guest_os()
        self._process_arch()
        self._process_machine_type()
        self._process_restart_vm()
        self._process_restore_image()
        self._process_mem()
        self._process_tcpdump()
        self._process_no_filter()
        self._process_only_filter()
        self._process_qemu_img()
        self._process_bridge_mode()
        self._process_only_type_specific()

    def _process_backend_specific_options(self, vt_type):
        """
        Calls for processing of backend specific options
        """
        backend_specific_options = {
            'qemu': self._process_qemu_specific_options,
            'lvsb': self._process_lvsb_specific_options,
            'openvswitch': self._process_qemu_specific_options,
            'libvirt': self._process_libvirt_specific_options,
            'spice': self._process_spice_specific_options,
            }
        specific_opts = backend_specific_options.get(vt_type, None)
        if specific_opts is not None:
            specific_opts()

    def _process_spice_specific_options(self):
        """
        Calls for processing all options specific to spice test
        """
        # We can call here for self._process_qemu_specific_options()
        # to process some --options, but let SpiceQA tests will be independent
        set_opt(self.config, 'no_downloads', True)

    def _process_options(self):
        """
        Process the options given in the command line.
        """
        cfg = None
        vt_type_setting = 'option --vt-type'
        vt_config_setting = 'option --vt-config'

        vt_type = get_opt(self.config, 'vt.type')
        vt_config = get_opt(self.config, 'vt.config')

        if (not vt_type) and (not vt_config):
            raise ValueError("No %s or %s specified" %
                             (vt_type_setting, vt_config_setting))

        if vt_type:
            if vt_type not in SUPPORTED_TEST_TYPES:
                raise ValueError("Invalid %s %s. Valid values: %s. "
                                 % (vt_type_setting,
                                    vt_type,
                                    " ".join(SUPPORTED_TEST_TYPES)))

        self.cartesian_parser = cartesian_config.Parser(debug=False)

        if vt_config:
            cfg = os.path.abspath(vt_config)
            self.cartesian_parser.parse_file(cfg)
        elif get_opt(self.config, 'vt.filter.default_filters'):
            cfg = data_dir.get_backend_cfg_path(vt_type,
                                                'tests-shared.cfg')
            self.cartesian_parser.parse_file(cfg)
            for arg in ('no_9p_export', 'no_virtio_rng', 'no_pci_assignable',
                        'smallpages', 'default_bios', 'bridge'):
                if arg not in get_opt(self.config, 'vt.filter.default_filters'):
                    self.cartesian_parser.only_filter(arg)
            if 'image_backend' not in get_opt(self.config, 'vt.filter.default_filters'):
                self.cartesian_parser.only_filter('(image_backend='
                                                  'filesystem)')
            if 'multihost' not in get_opt(self.config, 'vt.filter.default_filters'):
                self.cartesian_parser.no_filter('multihost')
        else:
            cfg = data_dir.get_backend_cfg_path(vt_type, 'tests.cfg')
            self.cartesian_parser.parse_file(cfg)

        if vt_type != 'lvsb':
            self._process_general_options()

        self._process_backend_specific_options(vt_type)
        self._process_extra_params()

    def get_parser(self):
        self._process_options()
        return self.cartesian_parser



"""
memory device support class(es)

"""

from virttest.libvirt_xml import accessors
from virttest.libvirt_xml.devices import base
from virttest.libvirt_xml.devices import librarian


class Memory(base.UntypedDeviceBase):

    __slots__ = ('mem_model', 'target', 'source', 'address', 'mem_discard',
                 'mem_access', 'alias', 'uuid')

    def __init__(self, virsh_instance=base.base.virsh):
        accessors.XMLAttribute('mem_model', self,
                               parent_xpath='/',
                               tag_name='memory',
                               attribute='model')
        accessors.XMLAttribute('mem_discard', self,
                               parent_xpath='/',
                               tag_name='memory',
                               attribute='discard')
        accessors.XMLAttribute('mem_access', self,
                               parent_xpath='/',
                               tag_name='memory',
                               attribute='access')
        accessors.XMLElementText("uuid", self,
                                 parent_xpath='/',
                                 tag_name='uuid')
        accessors.XMLElementNest('target', self, parent_xpath='/',
                                 tag_name='target', subclass=self.Target,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        accessors.XMLElementNest('source', self, parent_xpath='/',
                                 tag_name='source', subclass=self.Source,
                                 subclass_dargs={
                                     'virsh_instance': virsh_instance})
        accessors.XMLElementNest('address', self, parent_xpath='/',
                                 tag_name='address', subclass=self.Address,
                                 subclass_dargs={
                                     'type_name': 'dimm',
                                     'virsh_instance': virsh_instance})
        accessors.XMLElementDict('alias', self, parent_xpath='/',
                                 tag_name='alias')
        super(Memory, self).__init__(device_tag='memory',
                                     virsh_instance=virsh_instance)
        self.xml = '<memory/>'

    Address = librarian.get('address')

    class Target(base.base.LibvirtXMLBase):

        """
        Memory target xml class.

        Properties:

        size, node:
            int.
        size_unit:
            string.
        """
        __slots__ = ('size', 'size_unit', 'node', 'label')

        def __init__(self, virsh_instance=base.base.virsh):
            accessors.XMLElementInt('size',
                                    self, parent_xpath='/',
                                    tag_name='size')
            accessors.XMLAttribute(property_name="size_unit",
                                   libvirtxml=self,
                                   forbidden=None,
                                   parent_xpath='/',
                                   tag_name='size',
                                   attribute='unit')
            accessors.XMLElementInt('node',
                                    self, parent_xpath='/',
                                    tag_name='node')
            accessors.XMLElementNest('label', self, parent_xpath='/',
                                     tag_name='label', subclass=self.Label,
                                     subclass_dargs={
                                         'virsh_instance': virsh_instance})
            super(self.__class__, self).__init__(virsh_instance=virsh_instance)
            self.xml = '<target/>'

        class Label(base.base.LibvirtXMLBase):

            """
            Memory target label xml class.

            Properties:

            size:
                int.
            size_unit:
                string.
            """
            __slots__ = ('size', 'size_unit')

            def __init__(self, virsh_instance=base.base.virsh):
                accessors.XMLElementInt('size',
                                        self, parent_xpath='/',
                                        tag_name='size')
                accessors.XMLAttribute(property_name="size_unit",
                                       libvirtxml=self,
                                       forbidden=None,
                                       parent_xpath='/',
                                       tag_name='size',
                                       attribute='unit')
                super(self.__class__, self).__init__(virsh_instance=virsh_instance)
                self.xml = '<label/>'

    class Source(base.base.LibvirtXMLBase):

        """
        Memory source xml class.

        Properties:

        pagesize:
            int.
        pagesize_unit, nodemask:
            string.
        """
        __slots__ = ('pagesize', 'pagesize_unit', 'nodemask', 'path')

        def __init__(self, virsh_instance=base.base.virsh):
            accessors.XMLElementInt('pagesize',
                                    self, parent_xpath='/',
                                    tag_name='pagesize')
            accessors.XMLAttribute(property_name="pagesize_unit",
                                   libvirtxml=self,
                                   forbidden=None,
                                   parent_xpath='/',
                                   tag_name='pagesize',
                                   attribute='unit')
            accessors.XMLElementText('nodemask',
                                     self, parent_xpath='/',
                                     tag_name='nodemask')
            accessors.XMLElementText('path',
                                     self, parent_xpath='/',
                                     tag_name='path')
            super(self.__class__, self).__init__(virsh_instance=virsh_instance)
            self.xml = '<source/>'

    def new_mem_address(self, type_name='dimm', **dargs):
        """
        Return a new disk Address instance and set properties from dargs
        """
        new_one = self.Address(type_name=type_name, virsh_instance=self.virsh)
        for key, value in list(dargs.items()):
            setattr(new_one, key, value)
        return new_one



import unittest

try:
    from unittest import mock
except ImportError:
    import mock

from virttest import utils_kernel_module

# Test values
some_module_name = 'skvm'
some_module_param = 'force_emulation_prefix'
some_module_val = 'N'
some_module_config = {some_module_param: some_module_val,
                      'halt_poll_ns_shrink': '0'}
some_module_params = "%s=%s" % (some_module_param, some_module_val)

# Mocks
getstatusoutput_ok = mock.Mock(return_value=(0, ""))


@mock.patch.object(utils_kernel_module.os, 'listdir', return_value=[some_module_param])
@mock.patch.object(utils_kernel_module, 'open', mock.mock_open(read_data=some_module_val + '\n'))
@mock.patch.object(utils_kernel_module.process, 'getstatusoutput', getstatusoutput_ok)
class TestUnloadModule(unittest.TestCase):
    """
    Tests the reload_module method
    """

    def tearDown(self):
        getstatusoutput_ok.reset_mock()

    def assertUnloaded(self):
        getstatusoutput_ok.assert_called_once()

    def assertNoUnload(self):
        getstatusoutput_ok.assert_not_called()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc1(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.unload_module()
        self.assertUnloaded()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=False)
    def test_tc2(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.unload_module()
        self.assertNoUnload()


@mock.patch.object(utils_kernel_module.os, 'listdir', return_value=[some_module_param])
@mock.patch.object(utils_kernel_module, 'open', mock.mock_open(read_data=some_module_val + '\n'))
@mock.patch.object(utils_kernel_module.process, 'getstatusoutput', getstatusoutput_ok)
class TestReloadModule(unittest.TestCase):
    """
    Tests the reload_module method
    """

    def tearDown(self):
        getstatusoutput_ok.reset_mock()

    def assertLoadedWith(self, params):
        self.assertTrue(getstatusoutput_ok.called)
        cmd = getstatusoutput_ok.call_args[0][0]
        if params != "":
            self.assertTrue(params in cmd)
        else:
            self.assertTrue(cmd.endswith(some_module_name))

    def assertNotLoaded(self):
        getstatusoutput_ok.assert_not_called()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=False)
    def test_tc1(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.reload_module(force=True, params=some_module_params)
        self.assertLoadedWith(some_module_params)

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=False)
    def test_tc2(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.reload_module(force=False, params=some_module_params)
        self.assertNotLoaded()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc3(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.reload_module(force=True, params=some_module_params)
        self.assertLoadedWith(some_module_params)

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc3_empty(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.reload_module(force=True, params="")
        self.assertLoadedWith("")

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc4(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.reload_module(force=True, params="key=value")
        self.assertLoadedWith("key=value")

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc5(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.reload_module(force=False, params=some_module_params)
        self.assertNotLoaded()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc5_empty(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.reload_module(force=False, params="")
        self.assertNotLoaded()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc6(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.reload_module(force=False, params="key=value")
        self.assertLoadedWith("key=value")

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc6_partial(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.assertEqual(some_module_params, self.handler.config_backup)
        self.handler.reload_module(force=False, params=some_module_params + " key=value")
        self.assertLoadedWith("key=value")


@mock.patch.object(utils_kernel_module.process, 'getstatusoutput', getstatusoutput_ok)
@mock.patch.object(utils_kernel_module.os, 'listdir', return_value=[some_module_param])
@mock.patch.object(utils_kernel_module, 'open', mock.mock_open(read_data=some_module_val + '\n'))
class TestInit(unittest.TestCase):
    """
    Tests if module status is backed up correctly
    """

    def tearDown(self):
        getstatusoutput_ok.reset_mock()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_reloaded(self, *mocks):
        handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.assertTrue(handler.was_loaded)
        self.assertEqual(some_module_params, handler.config_backup)

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=False)
    def test_not_reloaded(self, *mocks):
        handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.assertFalse(handler.was_loaded)
        self.assertIsNone(handler.config_backup)


@mock.patch.object(utils_kernel_module.os, 'listdir', return_value=[some_module_param])
@mock.patch.object(utils_kernel_module, 'open', mock.mock_open(read_data=some_module_val + '\n'))
@mock.patch.object(utils_kernel_module.process, 'getstatusoutput', getstatusoutput_ok)
class TestRestore(unittest.TestCase):
    """
    Tests the restore method
    """

    def tearDown(self):
        getstatusoutput_ok.reset_mock()

    def assertRestored(self, orig_params, cur_params):
        self.assertTrue(getstatusoutput_ok.called)
        self.assertEqual(orig_params, cur_params)

    def assertNoRestore(self):
        getstatusoutput_ok.assert_not_called()

    def assertUnloaded(self):
        getstatusoutput_ok.assert_called_once()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc1(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        orig_config = self.handler.config_backup
        self.handler.reload_module(True, "key=value")
        self.handler.restore()
        cur_config = self.handler.current_config
        self.assertRestored(orig_config, cur_config)

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc1_reload_twice(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        orig_config = self.handler.config_backup
        self.handler.reload_module(True, "key=value")
        self.handler.reload_module(True, "key1=value1")
        self.handler.restore()
        cur_config = self.handler.current_config
        self.assertRestored(orig_config, cur_config)

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=True)
    def test_tc2(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.restore()
        self.assertNoRestore()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=False)
    def test_tc3(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.reload_module(True, "key=value")
        self.handler.restore()
        self.assertUnloaded()

    @mock.patch.object(utils_kernel_module.os.path, 'exists', return_value=False)
    def test_tc4(self, *mocks):
        self.handler = utils_kernel_module.KernelModuleHandler(some_module_name)
        self.handler.restore()
        self.assertNoRestore()


@mock.patch.object(utils_kernel_module.KernelModuleHandler, '__init__', return_value=None)
@mock.patch.object(utils_kernel_module.KernelModuleHandler, 'reload_module', return_value=None)
class TestReload(unittest.TestCase):
    """
    Tests the module global reload method
    """

    def test_empty_params_1(self, *mocks):
        self.assertIsNone(utils_kernel_module.reload(some_module_name, force=False, params=""))

    def test_empty_params_2(self, *mocks):
        self.assertIsNotNone(utils_kernel_module.reload(some_module_name, force=True, params=""))

    def test_non_empty_params_1(self, *mocks):
        self.assertIsNotNone(utils_kernel_module.reload(some_module_name, force=False, params=some_module_params))

    def test_non_empty_params_2(self, *mocks):
        self.assertIsNotNone(utils_kernel_module.reload(some_module_name, force=True, params=some_module_params))


if __name__ == '__main__':
    unittest.main()



"""
Shared code for tests that need to control libvirt on remote
"""

import logging

from avocado.core import exceptions

from virttest import remote

from virttest.utils_test import libvirt


def update_remote_file(params, value,
                       file_path='/etc/libvirt/libvirtd.conf',
                       restart_libvirt=True):
    """
    Update file on remote and restart libvirtd if needed

    :param param: Test run params
    :param value: The value to update
    :param file_path: The path of the file
    :param restart_libvirt: True to restart libvirtd
    :return: remote.RemoteFile object

    """
    try:
        tmp_value = eval(value)
        logging.debug("Update file {} with: {}".format(file_path, value))
        remote_ip = params.get("server_ip", params.get("remote_ip"))
        remote_pwd = params.get("server_pwd", params.get("remote_pwd"))
        remote_user = params.get("server_user", params.get("remote_user"))

        remote_file_obj = remote.RemoteFile(address=remote_ip, client='scp',
                                            username=remote_user,
                                            password=remote_pwd,
                                            port='22', remote_path=file_path)
        remote_file_obj.sub_else_add(tmp_value)
        if restart_libvirt:
            libvirt.remotely_control_libvirtd(remote_ip, remote_user,
                                              remote_pwd, action='restart',
                                              status_error='no')
        return remote_file_obj
    except Exception as err:
        raise exceptions.TestFail("Unable to update {}: {}"
                                  .format(file_path, err))



#!/usr/bin/python

import os


mem_fd = open("/proc/meminfo", 'r')
contents = mem_fd.readlines()
mem_fd.close()
freemem = 0
for content in contents:
    if content.count("MemFree"):
        freemem = int(content.split()[1])
        break

int_size = 4
int_count = int(freemem * 256 * 0.8)

occupy_script = """#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

main()
{
  int *p=NULL;
  int count=%s;
  int size=%s;
  p=(int *)calloc(count, size);
  if(p==NULL)
  {
     exit(1);
  }
  int i;
  for(i=0;i<count-10;i++)
  {
    p[i]=0;
  }
  mlock(p, count*size);
  sleep(120);
  munlock(p, count*size);
  free(p);
}
""" % (int_count, int_size)

occupy_fd = open("/tmp/duplicate_pages.c", 'w')
occupy_fd.write(occupy_script)
occupy_fd.close()

# Compile
os.popen("cd /tmp;gcc -o duplicate_pages duplicate_pages.c")
os.popen("cd /tmp;./duplicate_pages &")



"""
lease device support class(es)

http://libvirt.org/formatdomain.html#elementsLease
"""

from virttest.libvirt_xml.devices import base
from virttest.libvirt_xml import accessors


class Lease(base.UntypedDeviceBase):
    __slots__ = ('lockspace', 'key', 'target')

    def __init__(self, virsh_instance=base.base.virsh):
        # example for lease : lockspace, key and target
        #
        # <lease>
        #   <lockspace>somearea</lockspace>
        #   <key>somekey</key>
        #   <target path='/some/lease/path' offset='1024'/>
        # </lease>
        accessors.XMLElementText('lockspace', self,
                                 parent_xpath='/',
                                 tag_name='lockspace')
        accessors.XMLElementText('key', self,
                                 parent_xpath='/',
                                 tag_name='key')
        accessors.XMLElementDict('target', self, parent_xpath='/',
                                 tag_name='target')
        super(Lease, self).__init__(device_tag='lease',
                                    virsh_instance=virsh_instance)



import logging


# implementation follows

logger = logging.getLogger()


_caller_code_to_skip_in_logging_stack = set()


def do_not_report_as_logging_caller(func):
    """Decorator to annotate functions we will tell logging not to log."""
    # These are not the droids you are looking for.
    # You may go about your business.
    _caller_code_to_skip_in_logging_stack.add(func.__code__)
    return func


class LoggingFile(object):

    """
    File-like object that will receive messages pass them to the logging
    infrastructure in an appropriate way.
    """

    def __init__(self, prefix='', level=logging.DEBUG,
                 logger=logging.getLogger()):
        """
        :param prefix - The prefix for each line logged by this object.
        """
        self._prefix = prefix
        self._level = level
        self._buffer = []
        self._logger = logger

    @do_not_report_as_logging_caller
    def write(self, data):
        """"
        Writes data only if it constitutes a whole line. If it's not the case,
        store it in a buffer and wait until we have a complete line.
        :param data - Raw data (a string) that will be processed.
        """
        # splitlines() discards a trailing blank line, so use split() instead
        data_lines = data.split('\n')
        if len(data_lines) > 1:
            self._buffer.append(data_lines[0])
            self._flush_buffer()
        for line in data_lines[1:-1]:
            self._log_line(line)
        if data_lines[-1]:
            self._buffer.append(data_lines[-1])

    @do_not_report_as_logging_caller
    def writelines(self, lines):
        """"
        Writes itertable of lines

        :param lines: An iterable of strings that will be processed.
        """
        for data in lines:
            self.write(data)

    @do_not_report_as_logging_caller
    def _log_line(self, line):
        """
        Passes lines of output to the logging module.
        """
        self._logger.log(self._level, self._prefix + line)

    @do_not_report_as_logging_caller
    def _flush_buffer(self):
        if self._buffer:
            self._log_line(''.join(self._buffer))
            self._buffer = []

    @do_not_report_as_logging_caller
    def flush(self):
        self._flush_buffer()

    def isatty(self):
        return False



"""
Package utility for manage package operation on host
"""
import logging
import aexpect

from avocado.core import exceptions
from avocado.utils import software_manager
from six import string_types

from virttest import utils_misc
from virttest import vt_console

PACKAGE_MANAGERS = ['apt-get',
                    'yum',
                    'zypper',
                    'dnf']

PKG_MGR_TIMEOUT = 300


class RemotePackageMgr(object):
    """
    The remote package manage class
    """

    def __init__(self, session, pkg):
        """
        :param session: session object
        :param pkg: package name or list
        """
        if not isinstance(session,
                          (aexpect.ShellSession,
                           aexpect.Expect,
                           vt_console.ConsoleSession)):
            raise exceptions.TestError("Parameters exception on session")
        if not isinstance(pkg, list):
            if not isinstance(pkg, string_types):
                raise exceptions.TestError("pkg %s must be list or str" % pkg)
            else:
                self.pkg_list = [pkg, ]
        else:
            self.pkg_list = pkg
        self.package_manager = None
        self.cmd = None
        self.query_cmd = None
        self.install_cmd = None
        self.remove_cmd = None
        self.clean_cmd = None
        self.session = session

        # Inspect and set package manager
        for pkg_mgr in PACKAGE_MANAGERS:
            cmd = 'which ' + pkg_mgr
            if not self.session.cmd_status(cmd):
                self.package_manager = pkg_mgr
                break

        if not self.package_manager:
            raise exceptions.TestError("Package manager not in %s" %
                                       PACKAGE_MANAGERS)
        elif self.package_manager == 'apt-get':
            self.query_cmd = "dpkg -s "
            self.remove_cmd = "apt-get --purge remove -y "
            self.install_cmd = "apt-get install -y "
            self.clean_cmd = "apt-get clean"
        else:
            self.query_cmd = "rpm -q "
            self.remove_cmd = self.package_manager + " remove -y "
            self.install_cmd = self.package_manager + " install -y "
            self.clean_cmd = self.package_manager + " clean all"

    def clean(self):
        """
        Run clean command to refresh repo db

        :return: True or False
        """
        return not self.session.cmd_status(self.clean_cmd, timeout=PKG_MGR_TIMEOUT)

    def is_installed(self, pkg_name):
        """
        Check the package installed status

        :param pkg_name: package name
        :return: True or False
        """
        cmd = self.query_cmd + pkg_name
        return not self.session.cmd_status(cmd, timeout=PKG_MGR_TIMEOUT)

    def operate(self, timeout, default_status, internal_timeout=2):
        """
        Run command and return status

        :param timeout: command timeout
        :param default_status: package default installed status
        :param internal_timeout: internal_timeout to pass to cmd_status
        :return: True of False
        """
        for pkg in self.pkg_list:
            need = False
            if '*' not in pkg:
                if self.is_installed(pkg) == default_status:
                    need = True
            else:
                need = True
            if need:
                cmd = self.cmd + pkg
                if self.session.cmd_status(cmd, timeout, internal_timeout):
                    # Try to clean the repo db and re-try installation
                    if not self.clean():
                        logging.error("Package %s was broken",
                                      self.package_manager)
                        return False
                    if self.session.cmd_status(cmd, timeout):
                        logging.error("Operate %s with %s failed", pkg, cmd)
                        return False
        return True

    def install(self, timeout=PKG_MGR_TIMEOUT):
        """
        Use package manager install packages

        :param timeout: install timeout
        :return: if install succeed return True, else False
        """
        self.cmd = self.install_cmd
        return self.operate(timeout, False)

    def remove(self, timeout=PKG_MGR_TIMEOUT):
        """
        Use package manager remove packages

        :param timeout: remove timeout
        :return: if remove succeed return True, else False
        """
        self.cmd = self.remove_cmd
        return self.operate(timeout, True)


class LocalPackageMgr(software_manager.SoftwareManager):

    """
    Local package manage class
    """

    def __init__(self, pkg):
        """
        :param pkg: package name or list
        """
        if not isinstance(pkg, list):
            if not isinstance(pkg, string_types):
                raise exceptions.TestError("pkg %s must be list or str" % pkg)
            else:
                self.pkg_list = [pkg, ]
        else:
            self.pkg_list = pkg
        super(LocalPackageMgr, self).__init__()
        self.func = None

    def operate(self, default_status):
        """
        Use package manager to operate packages

        :param default_status: package default installed status
        """
        for pkg in self.pkg_list:
            need = False
            if '*' not in pkg:
                if self.check_installed(pkg) == default_status:
                    need = True
            else:
                need = True
            if need:
                if not self.func(pkg):
                    logging.error("Operate %s on host failed", pkg)
                    return False
        return True

    def install(self):
        """
        Use package manager install packages

        :return: if install succeed return True, else False
        """
        self.func = super(LocalPackageMgr, self).__getattr__('install')
        return self.operate(False)

    def remove(self):
        """
        Use package manager remove packages

        :return: if remove succeed return True, else False
        """
        self.func = super(LocalPackageMgr, self).__getattr__('remove')
        return self.operate(True)


def package_manager(session, pkg):
    """
    Package manager function

    :param session: session object
    :param pkg: pkg name or list
    :return: package manager class object
    """
    if session:
        mgr = RemotePackageMgr(session, pkg)
    else:
        mgr = LocalPackageMgr(pkg)
    return mgr


def package_install(pkg, session=None, timeout=PKG_MGR_TIMEOUT):
    """
    Try to install packages on system with package manager.

    :param pkg: package name or list of packages
    :param session: session Object
    :param timeout: timeout for install with session
    :return: True if all packages installed, False if any error
    """
    mgr = package_manager(session, pkg)
    return utils_misc.wait_for(mgr.install, timeout)


def package_remove(pkg, session=None, timeout=PKG_MGR_TIMEOUT):
    """
    Try to remove packages on system with package manager.

    :param pkg: package name or list of packages
    :param session: session Object
    :param timeout: timeout for remove with session
    :return: True if all packages removed, False if any error
    """
    mgr = package_manager(session, pkg)
    return utils_misc.wait_for(mgr.remove, timeout)



import platform
ARCH = platform.machine()
DEFAULT_MACHINE_TYPE = None

if ARCH in ('ppc64', 'ppc64le'):
    DEFAULT_MACHINE_TYPE = "pseries"
elif ARCH in ('x86_64', 'i386'):
    DEFAULT_MACHINE_TYPE = "i440fx"
else:
    # TODO: Handle other supported archs
    pass


def get_default_guest_os_info():
    """
    Gets the default asset and variant information
    TODO: Check for the ARCH and choose corresponding default asset
    """
    default_asset = 'jeos-27-'
    default_asset = '%s%s' % (default_asset, ARCH)
    return {'asset': default_asset, 'variant': 'JeOS.27'}


DEFAULT_GUEST_OS = get_default_guest_os_info()['variant']

__all__ = ['DEFAULT_MACHINE_TYPE', 'DEFAULT_GUEST_OS',
           'get_default_guest_os_info']
